@@ -35,58 +35,71 @@ const (
 )
 
 // WithContainerDisk specifies the disk name and the name of the container image to be used.
-func WithContainerDisk(diskName, imageName string) Option {
+func WithContainerDisk(diskName, imageName string, options ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio))
-		addVolume(vmi, newContainerVolume(diskName, imageName))
+		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio, options...))
+		addVolume(vmi, newContainerVolume(diskName, imageName, ""))
+	}
+}
+
+// WithContainerDiskAndPullPolicy specifies the disk name, the name of the container image and Pull Policy to be used.
+func WithContainerDiskAndPullPolicy(diskName, imageName string, imagePullPolicy k8sv1.PullPolicy, diskOpts ...DiskOption) Option {
+	return func(vmi *v1.VirtualMachineInstance) {
+		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio, diskOpts...))
+		addVolume(vmi, newContainerVolume(diskName, imageName, imagePullPolicy))
 	}
 }
 
 // WithContainerSATADisk specifies the disk name and the name of the container image to be used.
-func WithContainerSATADisk(diskName, imageName string) Option {
+func WithContainerSATADisk(diskName, imageName string, diskOpts ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, v1.DiskBusSATA))
-		addVolume(vmi, newContainerVolume(diskName, imageName))
+		addDisk(vmi, newDisk(diskName, v1.DiskBusSATA, diskOpts...))
+		addVolume(vmi, newContainerVolume(diskName, imageName, ""))
 	}
 }
 
 // WithPersistentVolumeClaim specifies the name of the PersistentVolumeClaim to be used.
-func WithPersistentVolumeClaim(diskName, pvcName string) Option {
+func WithPersistentVolumeClaim(diskName, pvcName string, diskOpts ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio))
+		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio, diskOpts...))
 		addVolume(vmi, newPersistentVolumeClaimVolume(diskName, pvcName))
 	}
 }
 
 // WithEphemeralPersistentVolumeClaim specifies the name of the Ephemeral.PersistentVolumeClaim to be used.
-func WithEphemeralPersistentVolumeClaim(diskName, pvcName string) Option {
+func WithEphemeralPersistentVolumeClaim(diskName, pvcName string, diskOpts ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, v1.DiskBusSATA))
+		addDisk(vmi, newDisk(diskName, v1.DiskBusSATA, diskOpts...))
 		addVolume(vmi, newEphemeralPersistentVolumeClaimVolume(diskName, pvcName))
 	}
 }
 
 // WithDataVolume specifies the name of the DataVolume to be used.
-func WithDataVolume(diskName, pvcName string) Option {
+func WithDataVolume(diskName, pvcName string, diskOpts ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio))
+		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio, diskOpts...))
 		addVolume(vmi, newDataVolume(diskName, pvcName))
 	}
 }
 
 // WithEmptyDisk specifies the name of the EmptyDisk to be used.
-func WithEmptyDisk(diskName string, bus v1.DiskBus, capacity resource.Quantity) Option {
+func WithEmptyDisk(diskName string, bus v1.DiskBus, capacity resource.Quantity, diskOpts ...DiskOption) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newDisk(diskName, bus))
+		addDisk(vmi, newDisk(diskName, bus, diskOpts...))
 		addVolume(vmi, newEmptyDisk(diskName, capacity))
 	}
 }
 
 // WithCDRom specifies a CDRom drive backed by a PVC to be used.
 func WithCDRom(cdRomName string, bus v1.DiskBus, claimName string) Option {
+	return WithCDRomAndVolume(bus, newPersistentVolumeClaimVolume(cdRomName, claimName))
+}
+
+// WithCDRomAndVolume specifies a CDRom drive backed by given volume and given bus.
+func WithCDRomAndVolume(bus v1.DiskBus, volume v1.Volume) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
-		addDisk(vmi, newCDRom(cdRomName, bus))
-		addVolume(vmi, newPersistentVolumeClaimVolume(cdRomName, claimName))
+		addDisk(vmi, newCDRom(volume.Name, bus))
+		addVolume(vmi, volume)
 	}
 }
 
@@ -94,7 +107,7 @@ func WithCDRom(cdRomName string, bus v1.DiskBus, claimName string) Option {
 func WithEphemeralCDRom(cdRomName string, bus v1.DiskBus, claimName string) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
 		addDisk(vmi, newCDRom(cdRomName, bus))
-		addVolume(vmi, newContainerVolume(cdRomName, claimName))
+		addVolume(vmi, newContainerVolume(cdRomName, claimName, ""))
 	}
 }
 
@@ -121,26 +134,18 @@ func WithPersistentVolumeClaimLun(diskName, pvcName string, reservation bool) Op
 	}
 }
 
-func WithHostDisk(diskName, path string, diskType v1.HostDiskType, opts ...HostDiskOption) Option {
+func WithHostDisk(diskName, path string, diskType v1.HostDiskType) Option {
 	var capacity string
 	if diskType == v1.HostDiskExistsOrCreate {
 		capacity = defaultDiskSize
 	}
-	return WithHostDiskAndCapacity(diskName, path, diskType, capacity, opts...)
+	return WithHostDiskAndCapacity(diskName, path, diskType, capacity)
 }
 
-func WithHostDiskAndCapacity(diskName, path string, diskType v1.HostDiskType, capacity string, opts ...HostDiskOption) Option {
+func WithHostDiskAndCapacity(diskName, path string, diskType v1.HostDiskType, capacity string) Option {
 	return func(vmi *v1.VirtualMachineInstance) {
 		addDisk(vmi, newDisk(diskName, v1.DiskBusVirtio))
-		addVolume(vmi, newHostDisk(diskName, path, diskType, capacity, opts...))
-	}
-}
-
-type HostDiskOption func(v *v1.Volume)
-
-func WithSharedHostDisk(shared bool) HostDiskOption {
-	return func(v *v1.Volume) {
-		v.HostDisk.Shared = pointer.P(shared)
+		addVolume(vmi, newHostDisk(diskName, path, diskType, capacity))
 	}
 }
 
@@ -151,6 +156,12 @@ func WithIOThreadsPolicy(policy v1.IOThreadsPolicy) Option {
 	}
 }
 
+func WithIOThreads(iothreads v1.DiskIOThreads) Option {
+	return func(vmi *v1.VirtualMachineInstance) {
+		vmi.Spec.Domain.IOThreads = pointer.P(iothreads)
+	}
+}
+
 func addDisk(vmi *v1.VirtualMachineInstance, disk v1.Disk) {
 	if !diskExists(vmi, disk) {
 		vmi.Spec.Domain.Devices.Disks = append(vmi.Spec.Domain.Devices.Disks, disk)
@@ -207,8 +218,10 @@ func filesystemExists(vmi *v1.VirtualMachineInstance, filesystem v1.Filesystem)
 	return false
 }
 
-func newDisk(name string, bus v1.DiskBus) v1.Disk {
-	return v1.Disk{
+type DiskOption func(vm *v1.Disk)
+
+func newDisk(name string, bus v1.DiskBus, opts ...DiskOption) v1.Disk {
+	d := v1.Disk{
 		Name: name,
 		DiskDevice: v1.DiskDevice{
 			Disk: &v1.DiskTarget{
@@ -216,6 +229,17 @@ func newDisk(name string, bus v1.DiskBus) v1.Disk {
 			},
 		},
 	}
+	for _, f := range opts {
+		f(&d)
+	}
+
+	return d
+}
+
+func WithDedicatedIOThreads(enabled bool) DiskOption {
+	return func(d *v1.Disk) {
+		d.DedicatedIOThread = pointer.P(enabled)
+	}
 }
 
 func newCDRom(name string, bus v1.DiskBus) v1.Disk {
@@ -252,8 +276,8 @@ func newVolume(name string) v1.Volume {
 	return v1.Volume{Name: name}
 }
 
-func newContainerVolume(name, image string) v1.Volume {
-	return v1.Volume{
+func newContainerVolume(name, image string, imagePullPolicy k8sv1.PullPolicy) v1.Volume {
+	container := v1.Volume{
 		Name: name,
 		VolumeSource: v1.VolumeSource{
 			ContainerDisk: &v1.ContainerDiskSource{
@@ -261,6 +285,10 @@ func newContainerVolume(name, image string) v1.Volume {
 			},
 		},
 	}
+	if imagePullPolicy != "" {
+		container.ContainerDisk.ImagePullPolicy = imagePullPolicy
+	}
+	return container
 }
 
 func newPersistentVolumeClaimVolume(name, claimName string) v1.Volume {
@@ -311,25 +339,21 @@ func newEmptyDisk(name string, capacity resource.Quantity) v1.Volume {
 	}
 }
 
-func newHostDisk(name, path string, diskType v1.HostDiskType, capacity string, opts ...HostDiskOption) v1.Volume {
+func newHostDisk(name, path string, diskType v1.HostDiskType, capacity string) v1.Volume {
 	hostDisk := v1.HostDisk{
 		Path: path,
 		Type: diskType,
 	}
-	if diskType == v1.HostDiskExistsOrCreate {
-		hostDisk.Capacity = resource.MustParse(defaultDiskSize)
+
+	// Set capacity if provided
+	if capacity != "" {
+		hostDisk.Capacity = resource.MustParse(capacity)
 	}
 
-	v := v1.Volume{
+	return v1.Volume{
 		Name: name,
 		VolumeSource: v1.VolumeSource{
 			HostDisk: &hostDisk,
 		},
 	}
-
-	for _, f := range opts {
-		f(&v)
-	}
-
-	return v
 }
