@@ -58,6 +58,7 @@ var (
 			vmiAddresses,
 			vmiMigrationStartTime,
 			vmiMigrationEndTime,
+			vmiVnicInfo,
 		},
 		CollectCallback: vmiStatsCollectorCallback,
 	}
@@ -78,6 +79,8 @@ var (
 			"guest_os_kernel_release", "guest_os_machine", "guest_os_arch", "guest_os_name", "guest_os_version_id",
 			// State info
 			"evictable", "outdated",
+			// Pod info
+			"vmi_pod",
 		},
 	)
 
@@ -96,7 +99,7 @@ var (
 				"interface associated with the VMI in the 'address' label, and about the type of address, such as " +
 				"internal IP, in the 'type' label.",
 		},
-		[]string{"node", "namespace", "name", "vnic_name", "interface_name", "address", "type"},
+		[]string{"node", "namespace", "name", "network_name", "address", "type"},
 	)
 
 	vmiMigrationStartTime = operatormetrics.NewGaugeVec(
@@ -114,6 +117,15 @@ var (
 		},
 		[]string{"node", "namespace", "name", "migration_name", "status"},
 	)
+
+	vmiVnicInfo = operatormetrics.NewGaugeVec(
+		operatormetrics.MetricOpts{
+			Name: "kubevirt_vmi_vnic_info",
+			Help: "Details of VirtualMachineInstance (VMI) vNIC interfaces, such as vNIC name, binding type, " +
+				"network name, and binding name for each vNIC of a running instance.",
+		},
+		[]string{"name", "namespace", "vnic_name", "binding_type", "network", "binding_name"},
+	)
 )
 
 func vmiStatsCollectorCallback() []operatormetrics.CollectorResult {
@@ -140,6 +152,7 @@ func reportVmisStats(vmis []*k6tv1.VirtualMachineInstance) []operatormetrics.Col
 		crs = append(crs, getEvictionBlocker(vmi))
 		crs = append(crs, collectVMIInterfacesInfo(vmi)...)
 		crs = append(crs, collectVMIMigrationTime(vmi)...)
+		crs = append(crs, CollectVmisVnicInfo(vmi)...)
 	}
 
 	return crs
@@ -151,6 +164,7 @@ func collectVMIInfo(vmi *k6tv1.VirtualMachineInstance) operatormetrics.Collector
 	preference := getVMIPreference(vmi)
 	kernelRelease, guestOSMachineArch, name, versionID := getGuestOSInfo(vmi)
 	guestOSMachineType := getVMIMachine(vmi)
+	vmiPod := getVMIPod(vmi)
 
 	return operatormetrics.CollectorResult{
 		Metric: vmiInfo,
@@ -160,6 +174,7 @@ func collectVMIInfo(vmi *k6tv1.VirtualMachineInstance) operatormetrics.Collector
 			kernelRelease, guestOSMachineType, guestOSMachineArch, name, versionID,
 			strconv.FormatBool(isVMEvictable(vmi)),
 			strconv.FormatBool(isVMIOutdated(vmi)),
+			vmiPod,
 		},
 		Value: 1.0,
 	}
@@ -222,40 +237,54 @@ func getVMIMachine(vmi *k6tv1.VirtualMachineInstance) (guestOSMachineType string
 	return
 }
 
-func getVMIInstancetype(vmi *k6tv1.VirtualMachineInstance) string {
-	if instancetypeName, ok := vmi.Annotations[k6tv1.InstancetypeAnnotation]; ok {
-		key := types.NamespacedName{
-			Namespace: vmi.Namespace,
-			Name:      instancetypeName,
-		}
-		return fetchResourceName(key.String(), instancetypeMethods.InstancetypeStore)
+func getVMIPod(vmi *k6tv1.VirtualMachineInstance) string {
+	objs, err := kvPodInformer.GetIndexer().ByIndex(cache.NamespaceIndex, vmi.Namespace)
+	if err != nil {
+		return none
 	}
 
-	if clusterInstancetypeName, ok := vmi.Annotations[k6tv1.ClusterInstancetypeAnnotation]; ok {
-		return fetchResourceName(clusterInstancetypeName, instancetypeMethods.ClusterInstancetypeStore)
+	for _, obj := range objs {
+		pod, ok := obj.(*k8sv1.Pod)
+		if !ok {
+			continue
+		}
+
+		if pod.Labels["kubevirt.io/created-by"] == string(vmi.UID) && pod.Status.Phase == k8sv1.PodRunning {
+			if vmi.Status.NodeName == pod.Spec.NodeName {
+				return pod.Name
+			}
+		}
+	}
+
+	return none
+}
+
+func getVMIInstancetype(vmi *k6tv1.VirtualMachineInstance) string {
+	if instancetypeName, ok := vmi.Annotations[k6tv1.InstancetypeAnnotation]; ok {
+		return fetchResourceName(instancetypeName, instancetypeMethods.InstancetypeStore)
+	}
+
+	if instancetypeName, ok := vmi.Annotations[k6tv1.ClusterInstancetypeAnnotation]; ok {
+		return fetchResourceName(instancetypeName, instancetypeMethods.ClusterInstancetypeStore)
 	}
 
 	return none
 }
 
 func getVMIPreference(vmi *k6tv1.VirtualMachineInstance) string {
-	if preferenceName, ok := vmi.Annotations[k6tv1.PreferenceAnnotation]; ok {
-		key := types.NamespacedName{
-			Namespace: vmi.Namespace,
-			Name:      preferenceName,
-		}
-		return fetchResourceName(key.String(), instancetypeMethods.PreferenceStore)
+	if instancetypeName, ok := vmi.Annotations[k6tv1.PreferenceAnnotation]; ok {
+		return fetchResourceName(instancetypeName, instancetypeMethods.PreferenceStore)
 	}
 
-	if clusterPreferenceName, ok := vmi.Annotations[k6tv1.ClusterPreferenceAnnotation]; ok {
-		return fetchResourceName(clusterPreferenceName, instancetypeMethods.ClusterPreferenceStore)
+	if instancetypeName, ok := vmi.Annotations[k6tv1.ClusterPreferenceAnnotation]; ok {
+		return fetchResourceName(instancetypeName, instancetypeMethods.ClusterPreferenceStore)
 	}
 
 	return none
 }
 
-func fetchResourceName(key string, store cache.Store) string {
-	obj, ok, err := store.GetByKey(key)
+func fetchResourceName(name string, store cache.Store) string {
+	obj, ok, err := store.GetByKey(name)
 	if err != nil || !ok {
 		return other
 	}
@@ -267,7 +296,7 @@ func fetchResourceName(key string, store cache.Store) string {
 
 	vendorName := apiObj.GetLabels()[instancetypeVendorLabel]
 	if _, isWhitelisted := whitelistedInstanceTypeVendors[vendorName]; isWhitelisted {
-		return apiObj.GetName()
+		return name
 	}
 
 	return other
@@ -319,22 +348,15 @@ func collectVMIInterfacesInfo(vmi *k6tv1.VirtualMachineInstance) []operatormetri
 }
 
 func collectVMIInterfaceInfo(vmi *k6tv1.VirtualMachineInstance, iface k6tv1.VirtualMachineInstanceNetworkInterface) *operatormetrics.CollectorResult {
-	interfaceType := "ExternalInterface"
-
-	if iface.IP == "" {
-		if iface.Name == "" && iface.InterfaceName == "" {
-			// Avoid duplicate metric labels error
-			return nil
-		}
-
-		interfaceType = "SystemInterface"
+	if iface.IP == "" && iface.Name == "" {
+		return nil
 	}
 
 	return &operatormetrics.CollectorResult{
 		Metric: vmiAddresses,
 		Labels: []string{
 			vmi.Status.NodeName, vmi.Namespace, vmi.Name,
-			iface.Name, iface.InterfaceName, iface.IP, interfaceType,
+			iface.Name, iface.IP, "InternalIP",
 		},
 		Value: 1.0,
 	}
@@ -400,3 +422,34 @@ func getMigrationNameFromMigrationUID(namespace string, migrationUID types.UID)
 
 	return none
 }
+
+func CollectVmisVnicInfo(vmi *k6tv1.VirtualMachineInstance) []operatormetrics.CollectorResult {
+	var results []operatormetrics.CollectorResult
+
+	interfaces := vmi.Spec.Domain.Devices.Interfaces
+	networks := vmi.Spec.Networks
+
+	for _, iface := range interfaces {
+		bindingType, bindingName := getBinding(iface)
+		networkName, matchFound := getNetworkName(iface.Name, networks)
+
+		if !matchFound {
+			continue
+		}
+
+		results = append(results, operatormetrics.CollectorResult{
+			Metric: vmiVnicInfo,
+			Labels: []string{
+				vmi.Name,
+				vmi.Namespace,
+				iface.Name,
+				bindingType,
+				networkName,
+				bindingName,
+			},
+			Value: 1.0,
+		})
+	}
+
+	return results
+}
