@@ -138,16 +138,115 @@ var _ = Describe("VMI Stats Collector", func() {
 				Expect(cr).ToNot(BeNil())
 				Expect(cr.Metric.GetOpts().Name).To(ContainSubstring("kubevirt_vmi_info"))
 				Expect(cr.Value).To(BeEquivalentTo(1))
-				Expect(cr.Labels).To(HaveLen(16))
+				Expect(cr.Labels).To(HaveLen(17))
 
 				Expect(cr.Labels[3]).To(Equal(getVMIPhase(vmis[i])))
 				os, workload, flavor := getSystemInfoFromAnnotations(vmis[i].Annotations)
 				Expect(cr.Labels[4]).To(Equal(os))
 				Expect(cr.Labels[5]).To(Equal(workload))
 				Expect(cr.Labels[6]).To(Equal(flavor))
+				Expect(cr.Labels[16]).To(Equal(getVMIPod(vmis[i])))
 			}
 		})
 
+		It("should update the vmi_pod label correctly after migration", func() {
+			originalPod := &k8sv1.Pod{
+				ObjectMeta: newPodMetaForInformer("virt-launcher-originalpod", "test-ns", "test-vmi-uid"),
+				Spec: k8sv1.PodSpec{
+					NodeName: "initial-node",
+				},
+				Status: k8sv1.PodStatus{
+					Phase: k8sv1.PodRunning,
+				},
+			}
+			targetPod := &k8sv1.Pod{
+				ObjectMeta: newPodMetaForInformer("virt-launcher-targetpod", "test-ns", "test-vmi-uid"),
+				Spec: k8sv1.PodSpec{
+					NodeName: "target-node",
+				},
+				Status: k8sv1.PodStatus{
+					Phase: k8sv1.PodRunning,
+				},
+			}
+
+			_ = kvPodInformer.GetStore().Add(originalPod)
+			_ = kvPodInformer.GetStore().Add(targetPod)
+
+			vmi := &k6tv1.VirtualMachineInstance{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test-vmi",
+					Namespace: "test-ns",
+					UID:       "test-vmi-uid",
+				},
+				Status: k6tv1.VirtualMachineInstanceStatus{
+					Phase:    "Running",
+					NodeName: "target-node",
+					MigrationState: &k6tv1.VirtualMachineInstanceMigrationState{
+						TargetPod: "virt-launcher-targetpod",
+						Completed: true,
+						Failed:    false,
+					},
+				},
+			}
+
+			cr := collectVMIInfo(vmi)
+
+			Expect(cr).ToNot(BeNil())
+			Expect(cr.Metric.GetOpts().Name).To(ContainSubstring("kubevirt_vmi_info"))
+			Expect(cr.Value).To(BeEquivalentTo(1))
+			Expect(cr.Labels).To(HaveLen(17))
+			Expect(cr.Labels[16]).To(Equal("virt-launcher-targetpod"))
+		})
+
+		It("should return the original pod when migration failed", func() {
+			originalPod := &k8sv1.Pod{
+				ObjectMeta: newPodMetaForInformer("virt-launcher-originalpod", "test-ns", "test-vmi-uid"),
+				Spec: k8sv1.PodSpec{
+					NodeName: "initial-node",
+				},
+				Status: k8sv1.PodStatus{
+					Phase: k8sv1.PodRunning,
+				},
+			}
+			targetPod := &k8sv1.Pod{
+				ObjectMeta: newPodMetaForInformer("virt-launcher-targetpod", "test-ns", "test-vmi-uid"),
+				Spec: k8sv1.PodSpec{
+					NodeName: "target-node",
+				},
+				Status: k8sv1.PodStatus{
+					Phase: k8sv1.PodRunning,
+				},
+			}
+
+			_ = kvPodInformer.GetStore().Add(originalPod)
+			_ = kvPodInformer.GetStore().Add(targetPod)
+
+			vmi := &k6tv1.VirtualMachineInstance{
+				ObjectMeta: metav1.ObjectMeta{
+					Name:      "test-vmi",
+					Namespace: "test-ns",
+					UID:       "test-vmi-uid",
+				},
+				Status: k6tv1.VirtualMachineInstanceStatus{
+					Phase:    "Running",
+					NodeName: "initial-node",
+					MigrationState: &k6tv1.VirtualMachineInstanceMigrationState{
+						TargetPod: "virt-launcher-targetpod",
+						Completed: true,
+						Failed:    true,
+					},
+				},
+			}
+
+			cr := collectVMIInfo(vmi)
+
+			Expect(cr).ToNot(BeNil())
+			Expect(cr.Metric.GetOpts().Name).To(ContainSubstring("kubevirt_vmi_info"))
+			Expect(cr.Value).To(BeEquivalentTo(1))
+			Expect(cr.Labels).To(HaveLen(17))
+			Expect(cr.Labels[16]).To(Equal("virt-launcher-originalpod"))
+		})
+
 		DescribeTable("should show instance type value correctly", func(instanceTypeAnnotationKey string, instanceType string, expected string) {
 			annotations := map[string]string{}
 			if instanceType != "" {
@@ -158,7 +257,6 @@ var _ = Describe("VMI Stats Collector", func() {
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Name:        "running",
-						Namespace:   "test-ns",
 						Annotations: annotations,
 					},
 				},
@@ -174,7 +272,7 @@ var _ = Describe("VMI Stats Collector", func() {
 			Expect(cr).ToNot(BeNil())
 			Expect(cr.Metric.GetOpts().Name).To(ContainSubstring("kubevirt_vmi_info"))
 			Expect(cr.Value).To(BeEquivalentTo(1))
-			Expect(cr.Labels).To(HaveLen(16))
+			Expect(cr.Labels).To(HaveLen(17))
 			Expect(cr.Labels[7]).To(Equal(expected))
 		},
 			Entry("with no instance type expect <none>", k6tv1.InstancetypeAnnotation, "", "<none>"),
@@ -195,7 +293,6 @@ var _ = Describe("VMI Stats Collector", func() {
 				{
 					ObjectMeta: metav1.ObjectMeta{
 						Name:        "running",
-						Namespace:   "test-ns",
 						Annotations: annotations,
 					},
 				},
@@ -211,7 +308,7 @@ var _ = Describe("VMI Stats Collector", func() {
 
 			Expect(cr.Metric.GetOpts().Name).To(ContainSubstring("kubevirt_vmi_info"))
 			Expect(cr.Value).To(BeEquivalentTo(1))
-			Expect(cr.Labels).To(HaveLen(16))
+			Expect(cr.Labels).To(HaveLen(17))
 			Expect(cr.Labels[8]).To(Equal(expected))
 		},
 			Entry("with no preference expect <none>", k6tv1.PreferenceAnnotation, "", "<none>"),
@@ -270,14 +367,14 @@ var _ = Describe("VMI Stats Collector", func() {
 			}
 		},
 			Entry("no interfaces", [][]string{}),
-			Entry("one interface", [][]string{{"default", "", "192.168.1.2", "ExternalInterface"}}),
+			Entry("one interface", [][]string{{"default", "192.168.1.2", "InternalIP"}}),
 			Entry("two interfaces", [][]string{
-				{"networkA", "", "170.170.170.170", "ExternalInterface"},
-				{"networkB", "", "180.180.180.180", "ExternalInterface"},
+				{"networkA", "170.170.170.170", "InternalIP"},
+				{"networkB", "180.180.180.180", "InternalIP"},
 			}),
 		)
 
-		It("should create metric for interfaces with empty name, but with interface name", func() {
+		It("should not create metric for an interface with empty IP address", func() {
 			vmi := &k6tv1.VirtualMachineInstance{
 				ObjectMeta: metav1.ObjectMeta{
 					Namespace: "test-ns",
@@ -287,14 +384,16 @@ var _ = Describe("VMI Stats Collector", func() {
 					NodeName: "testNode",
 					Interfaces: []k6tv1.VirtualMachineInstanceNetworkInterface{
 						{
-							InfoSource:    "guest-agent",
-							InterfaceName: "br-int",
-							MAC:           "00:00:00:00:00:01",
+							InfoSource: "domain",
+							Name:       "default",
+							IP:         "10.244.140.86",
 						},
 						{
-							InfoSource:    "guest-agent",
-							InterfaceName: "ovs-system",
-							MAC:           "00:00:00:00:00:02",
+							InfoSource: "domain, multus-status",
+							Name:       "networkA",
+						},
+						{
+							InfoSource: "domain, multus-status",
 						},
 					},
 				},
@@ -302,45 +401,8 @@ var _ = Describe("VMI Stats Collector", func() {
 
 			metrics := collectVMIInterfacesInfo(vmi)
 			Expect(metrics).To(HaveLen(2))
-			Expect(metrics[0].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "", "br-int", "", "SystemInterface"}))
-			Expect(metrics[1].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "", "ovs-system", "", "SystemInterface"}))
-		})
-
-		It("should not create metric for an interface with empty IP address, name and interface name", func() {
-			vmi := &k6tv1.VirtualMachineInstance{
-				ObjectMeta: metav1.ObjectMeta{
-					Namespace: "test-ns",
-					Name:      "testvmi",
-				},
-				Status: k6tv1.VirtualMachineInstanceStatus{
-					NodeName: "testNode",
-					Interfaces: []k6tv1.VirtualMachineInstanceNetworkInterface{
-						{
-							InfoSource:    "domain, guest-agent, multus-status",
-							Name:          "net-0",
-							InterfaceName: "br-ex",
-							IP:            "10.11.126.126",
-						},
-						{
-							InfoSource:    "guest-agent",
-							InterfaceName: "ovs-system",
-						},
-						{
-							InfoSource:    "guest-agent",
-							InterfaceName: "br-int",
-						},
-						{
-							InfoSource: "guest-agent",
-						},
-					},
-				},
-			}
-
-			metrics := collectVMIInterfacesInfo(vmi)
-			Expect(metrics).To(HaveLen(3))
-			Expect(metrics[0].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "net-0", "br-ex", "10.11.126.126", "ExternalInterface"}))
-			Expect(metrics[1].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "", "ovs-system", "", "SystemInterface"}))
-			Expect(metrics[2].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "", "br-int", "", "SystemInterface"}))
+			Expect(metrics[0].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "default", "10.244.140.86", "InternalIP"}))
+			Expect(metrics[1].Labels).To(Equal([]string{"testNode", "test-ns", "testvmi", "networkA", "", "InternalIP"}))
 		})
 	})
 
@@ -418,15 +480,112 @@ var _ = Describe("VMI Stats Collector", func() {
 			})
 		})
 	})
+
+	Context("VMI vNIC info", func() {
+		It("should collect kubevirt_vmi_vnic_info metric with correct labels", func() {
+			vmi := &k6tv1.VirtualMachineInstance{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "test-ns",
+					Name:      "test-vmi",
+				},
+				Spec: k6tv1.VirtualMachineInstanceSpec{
+					Domain: k6tv1.DomainSpec{
+						Devices: k6tv1.Devices{
+							Interfaces: []k6tv1.Interface{
+								{
+									Name: "iface1",
+									InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+										Bridge: &k6tv1.InterfaceBridge{},
+									},
+								},
+								{
+									Name: "iface2",
+									InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+										Masquerade: &k6tv1.InterfaceMasquerade{},
+									},
+								},
+								{
+									Name: "iface3",
+									InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+										SRIOV: &k6tv1.InterfaceSRIOV{},
+									},
+								},
+								{
+									Name:    "iface4",
+									Binding: &k6tv1.PluginBinding{Name: "custom-plugin"},
+								},
+							},
+						},
+					},
+					Networks: []k6tv1.Network{
+						{
+							Name:          "iface1",
+							NetworkSource: k6tv1.NetworkSource{Pod: &k6tv1.PodNetwork{}},
+						},
+						{
+							Name:          "iface2",
+							NetworkSource: k6tv1.NetworkSource{Pod: &k6tv1.PodNetwork{}},
+						},
+						{
+							Name:          "iface3",
+							NetworkSource: k6tv1.NetworkSource{Multus: &k6tv1.MultusNetwork{NetworkName: "multus-net"}},
+						},
+						{
+							Name:          "iface4",
+							NetworkSource: k6tv1.NetworkSource{Multus: &k6tv1.MultusNetwork{NetworkName: "custom-net"}},
+						},
+					},
+				},
+			}
+
+			metrics := CollectVmisVnicInfo(vmi)
+			Expect(metrics).To(HaveLen(4))
+
+			Expect(metrics[0].Labels).To(Equal([]string{"test-vmi", "test-ns", "iface1", "core", "pod networking", "bridge"}))
+			Expect(metrics[1].Labels).To(Equal([]string{"test-vmi", "test-ns", "iface2", "core", "pod networking", "masquerade"}))
+			Expect(metrics[2].Labels).To(Equal([]string{"test-vmi", "test-ns", "iface3", "core", "multus-net", "sriov"}))
+			Expect(metrics[3].Labels).To(Equal([]string{"test-vmi", "test-ns", "iface4", "plugin", "custom-net", "custom-plugin"}))
+		})
+		It("should not collect kubevirt_vmi_vnic_info metric when interface name is not matching network name", func() {
+			vmi := &k6tv1.VirtualMachineInstance{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "test-ns",
+					Name:      "test-vmi",
+				},
+				Spec: k6tv1.VirtualMachineInstanceSpec{
+					Domain: k6tv1.DomainSpec{
+						Devices: k6tv1.Devices{
+							Interfaces: []k6tv1.Interface{
+								{
+									Name: "iface1",
+									InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+										Bridge: &k6tv1.InterfaceBridge{},
+									},
+								},
+							},
+						},
+					},
+					Networks: []k6tv1.Network{
+						{
+							Name:          "iface2",
+							NetworkSource: k6tv1.NetworkSource{Pod: &k6tv1.PodNetwork{}},
+						},
+					},
+				},
+			}
+
+			metrics := CollectVmisVnicInfo(vmi)
+			Expect(metrics).To(BeEmpty())
+		})
+	})
 })
 
 func interfacesFor(values [][]string) []k6tv1.VirtualMachineInstanceNetworkInterface {
 	interfaces := make([]k6tv1.VirtualMachineInstanceNetworkInterface, len(values))
 	for i, v := range values {
 		interfaces[i] = k6tv1.VirtualMachineInstanceNetworkInterface{
-			Name:          v[0],
-			InterfaceName: v[1],
-			IP:            v[2],
+			Name: v[0],
+			IP:   v[1],
 		}
 	}
 	return interfaces
@@ -462,17 +621,16 @@ func setupTestCollector() {
 	clusterInstanceTypeInformer, _ := testutils.NewFakeInformerFor(&instancetypev1beta1.VirtualMachineClusterInstancetype{})
 	preferenceInformer, _ := testutils.NewFakeInformerFor(&instancetypev1beta1.VirtualMachinePreference{})
 	clusterPreferenceInformer, _ := testutils.NewFakeInformerFor(&instancetypev1beta1.VirtualMachineClusterPreference{})
-	vmiMigrationInformer, _ = testutils.NewFakeInformerFor(&k6tv1.VirtualMachineInstanceMigration{})
 
 	_ = instanceTypeInformer.GetStore().Add(&instancetypev1beta1.VirtualMachineInstancetype{
-		ObjectMeta: newObjectMetaForInstancetypes("i-managed", "test-ns", "kubevirt.io"),
+		ObjectMeta: newObjectMetaForInstancetypes("i-managed", "kubevirt.io"),
 	})
 	_ = instanceTypeInformer.GetStore().Add(&instancetypev1beta1.VirtualMachineInstancetype{
-		ObjectMeta: newObjectMetaForInstancetypes("i-unmanaged", "test-ns", "some-user"),
+		ObjectMeta: newObjectMetaForInstancetypes("i-unmanaged", "some-user"),
 	})
 
 	_ = clusterInstanceTypeInformer.GetStore().Add(&instancetypev1beta1.VirtualMachineClusterInstancetype{
-		ObjectMeta: newObjectMetaForInstancetypes("ci-managed", "", "kubevirt.io"),
+		ObjectMeta: newObjectMetaForInstancetypes("ci-managed", "kubevirt.io"),
 		Spec: instancetypev1beta1.VirtualMachineInstancetypeSpec{
 			CPU: instancetypev1beta1.CPUInstancetype{
 				Guest: 2,
@@ -483,18 +641,18 @@ func setupTestCollector() {
 		},
 	})
 	_ = clusterInstanceTypeInformer.GetStore().Add(&instancetypev1beta1.VirtualMachineClusterInstancetype{
-		ObjectMeta: newObjectMetaForInstancetypes("ci-unmanaged", "", ""),
+		ObjectMeta: newObjectMetaForInstancetypes("ci-unmanaged", ""),
 	})
 
 	_ = preferenceInformer.GetStore().Add(&instancetypev1beta1.VirtualMachinePreference{
-		ObjectMeta: newObjectMetaForInstancetypes("p-managed", "test-ns", "kubevirt.io"),
+		ObjectMeta: newObjectMetaForInstancetypes("p-managed", "kubevirt.io"),
 	})
 	_ = preferenceInformer.GetStore().Add(&instancetypev1beta1.VirtualMachinePreference{
-		ObjectMeta: newObjectMetaForInstancetypes("p-unmanaged", "test-ns", "some-vendor.com"),
+		ObjectMeta: newObjectMetaForInstancetypes("p-unmanaged", "some-vendor.com"),
 	})
 
 	_ = clusterPreferenceInformer.GetStore().Add(&instancetypev1beta1.VirtualMachineClusterPreference{
-		ObjectMeta: newObjectMetaForInstancetypes("cp-managed", "", "kubevirt.io"),
+		ObjectMeta: newObjectMetaForInstancetypes("cp-managed", "kubevirt.io"),
 	})
 
 	instancetypeMethods = &instancetype.InstancetypeMethods{
@@ -504,7 +662,17 @@ func setupTestCollector() {
 		ClusterPreferenceStore:   clusterPreferenceInformer.GetStore(),
 	}
 
-	vmiMigrationInformer.GetStore().Add(&k6tv1.VirtualMachineInstanceMigration{
+	// Pod informer
+	kvPodInformer, _ = testutils.NewFakeInformerFor(&k8sv1.Pod{})
+
+	_ = kvPodInformer.GetStore().Add(&k8sv1.Pod{
+		ObjectMeta: newPodMetaForInformer("virt-launcher-testpod", "test-ns", "test-vmi-uid"),
+	})
+
+	// VMI Migration informer
+	vmiMigrationInformer, _ = testutils.NewFakeInformerFor(&k6tv1.VirtualMachineInstanceMigration{})
+
+	_ = vmiMigrationInformer.GetStore().Add(&k6tv1.VirtualMachineInstanceMigration{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      "test-migration",
 			Namespace: "test-ns",
@@ -513,15 +681,17 @@ func setupTestCollector() {
 	})
 }
 
-func newObjectMetaForInstancetypes(name, namespace, vendor string) metav1.ObjectMeta {
-	om := metav1.ObjectMeta{
+func newObjectMetaForInstancetypes(name, vendor string) metav1.ObjectMeta {
+	return metav1.ObjectMeta{
 		Name:   name,
 		Labels: map[string]string{instancetypeVendorLabel: vendor},
 	}
+}
 
-	if namespace != "" {
-		om.Namespace = namespace
+func newPodMetaForInformer(name, namespace, createdByUID string) metav1.ObjectMeta {
+	return metav1.ObjectMeta{
+		Name:      name,
+		Namespace: namespace,
+		Labels:    map[string]string{"kubevirt.io/created-by": createdByUID},
 	}
-
-	return om
 }
