@@ -23,13 +23,10 @@ import (
 	"strings"
 
 	"github.com/machadovilaca/operator-observability/pkg/operatormetrics"
-
 	k8sv1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
-	"k8s.io/apimachinery/pkg/types"
 
 	k6tv1 "kubevirt.io/api/core/v1"
-	instancetypeapi "kubevirt.io/api/instancetype"
 	"kubevirt.io/client-go/log"
 
 	"kubevirt.io/kubevirt/pkg/controller"
@@ -37,7 +34,7 @@ import (
 
 var (
 	vmStatsCollector = operatormetrics.Collector{
-		Metrics:         append(timestampMetrics, vmResourceRequests, vmResourceLimits, vmInfo, vmDiskAllocatedSize),
+		Metrics:         append(timestampMetrics, vmResourceRequests, vmResourceLimits, vmInfo, vmDiskAllocatedSize, vmCreationTimestamp, vmVnicInfo),
 		CollectCallback: vmStatsCollectorCallback,
 	}
 
@@ -170,6 +167,23 @@ var (
 		},
 		[]string{"name", "namespace", "persistentvolumeclaim", "volume_mode", "device"},
 	)
+
+	vmCreationTimestamp = operatormetrics.NewGaugeVec(
+		operatormetrics.MetricOpts{
+			Name: "kubevirt_vm_create_date_timestamp_seconds",
+			Help: "Virtual Machine creation timestamp.",
+		},
+		[]string{"name", "namespace"},
+	)
+
+	vmVnicInfo = operatormetrics.NewGaugeVec(
+		operatormetrics.MetricOpts{
+			Name: "kubevirt_vm_vnic_info",
+			Help: "Details of Virtual Machine (VM) vNIC interfaces, such as vNIC name, binding type, network name, " +
+				"and binding name for each vNIC defined in the VM's configuration.",
+		},
+		[]string{"name", "namespace", "vnic_name", "binding_type", "network", "binding_name"},
+	)
 )
 
 func vmStatsCollectorCallback() []operatormetrics.CollectorResult {
@@ -186,9 +200,12 @@ func vmStatsCollectorCallback() []operatormetrics.CollectorResult {
 	}
 
 	var results []operatormetrics.CollectorResult
+	results = append(results, CollectDiskAllocatedSize(vms)...)
 	results = append(results, CollectVMsInfo(vms)...)
 	results = append(results, CollectResourceRequestsAndLimits(vms)...)
 	results = append(results, reportVmsStats(vms)...)
+	results = append(results, collectVMCreationTimestamp(vms)...)
+	results = append(results, CollectVmsVnicInfo(vms)...)
 	return results
 }
 
@@ -214,7 +231,7 @@ func CollectVMsInfo(vms []*k6tv1.VirtualMachine) []operatormetrics.CollectorResu
 				vm.Name, vm.Namespace,
 				os, workload, flavor, machineType,
 				instanceType, preference,
-				string(vm.Status.PrintableStatus), getVMStatusGroup(vm.Status.PrintableStatus),
+				strings.ToLower(string(vm.Status.PrintableStatus)), getVMStatusGroup(vm.Status.PrintableStatus),
 			},
 			Value: 1.0,
 		})
@@ -230,15 +247,11 @@ func getVMInstancetype(vm *k6tv1.VirtualMachine) string {
 		return none
 	}
 
-	if strings.EqualFold(instancetype.Kind, instancetypeapi.SingularResourceName) {
-		key := types.NamespacedName{
-			Namespace: vm.Namespace,
-			Name:      instancetype.Name,
-		}
-		return fetchResourceName(key.String(), instancetypeMethods.InstancetypeStore)
+	if instancetype.Kind == "VirtualMachineInstancetype" {
+		return fetchResourceName(instancetype.Name, instancetypeMethods.InstancetypeStore)
 	}
 
-	if strings.EqualFold(instancetype.Kind, instancetypeapi.ClusterSingularResourceName) {
+	if instancetype.Kind == "VirtualMachineClusterInstancetype" {
 		return fetchResourceName(instancetype.Name, instancetypeMethods.ClusterInstancetypeStore)
 	}
 
@@ -252,15 +265,11 @@ func getVMPreference(vm *k6tv1.VirtualMachine) string {
 		return none
 	}
 
-	if strings.EqualFold(preference.Kind, instancetypeapi.SingularPreferenceResourceName) {
-		key := types.NamespacedName{
-			Namespace: vm.Namespace,
-			Name:      preference.Name,
-		}
-		return fetchResourceName(key.String(), instancetypeMethods.PreferenceStore)
+	if preference.Kind == "VirtualMachinePreference" {
+		return fetchResourceName(preference.Name, instancetypeMethods.PreferenceStore)
 	}
 
-	if strings.EqualFold(preference.Kind, instancetypeapi.ClusterSingularPreferenceResourceName) {
+	if preference.Kind == "VirtualMachineClusterPreference" {
 		return fetchResourceName(preference.Name, instancetypeMethods.ClusterPreferenceStore)
 	}
 
@@ -292,9 +301,6 @@ func CollectResourceRequestsAndLimits(vms []*k6tv1.VirtualMachine) []operatormet
 		vmCopy := vm.DeepCopy()
 		_ = instancetypeMethods.ApplyToVM(vmCopy)
 
-		// Disk size
-		results = append(results, CollectDiskAllocatedSize(vmCopy)...)
-
 		// Memory requests and limits from domain resources
 		results = append(results, collectMemoryResourceRequestsFromDomainResources(vmCopy)...)
 		results = append(results, collectMemoryResourceLimitsFromDomainResources(vmCopy)...)
@@ -549,13 +555,21 @@ func containsCondition(target k6tv1.VirtualMachineConditionType, elems []k6tv1.V
 	return false
 }
 
-func CollectDiskAllocatedSize(vm *k6tv1.VirtualMachine) []operatormetrics.CollectorResult {
+func CollectDiskAllocatedSize(vms []*k6tv1.VirtualMachine) []operatormetrics.CollectorResult {
 	var cr []operatormetrics.CollectorResult
 
-	if vm.Spec.Template == nil {
-		return cr
+	for _, vm := range vms {
+		if vm.Spec.Template != nil {
+			cr = append(cr, collectDiskMetricsFromPVC(vm)...)
+		}
 	}
 
+	return cr
+}
+
+func collectDiskMetricsFromPVC(vm *k6tv1.VirtualMachine) []operatormetrics.CollectorResult {
+	var cr []operatormetrics.CollectorResult
+
 	for _, vol := range vm.Spec.Template.Spec.Volumes {
 		pvcName, diskName, isDataVolume := getPVCAndDiskName(vol)
 		if pvcName == "" {
@@ -626,9 +640,102 @@ func getSizeFromDataVolumeTemplates(vm *k6tv1.VirtualMachine, dataVolumeName str
 		if dvTemplate.Name == dataVolumeName {
 			if dvTemplate.Spec.PVC != nil {
 				return dvTemplate.Spec.PVC.Resources.Requests.Storage()
-			} else if dvTemplate.Spec.Storage != nil {
-				return dvTemplate.Spec.Storage.Resources.Requests.Storage()
 			}
+
+			break
+		}
+	}
+
+	return nil
+}
+
+func collectVMCreationTimestamp(vms []*k6tv1.VirtualMachine) []operatormetrics.CollectorResult {
+	var cr []operatormetrics.CollectorResult
+
+	for _, vm := range vms {
+		if !vm.CreationTimestamp.IsZero() {
+			cr = append(cr, operatormetrics.CollectorResult{
+				Metric: vmCreationTimestamp,
+				Labels: []string{vm.Name, vm.Namespace},
+				Value:  float64(vm.CreationTimestamp.Unix()),
+			})
+		}
+	}
+
+	return cr
+}
+
+func CollectVmsVnicInfo(vms []*k6tv1.VirtualMachine) []operatormetrics.CollectorResult {
+	var results []operatormetrics.CollectorResult
+
+	for _, vm := range vms {
+		if vm.Spec.Template == nil || vm.Spec.Template.Spec.Domain.Devices.Interfaces == nil {
+			continue
+		}
+
+		interfaces := vm.Spec.Template.Spec.Domain.Devices.Interfaces
+		networks := vm.Spec.Template.Spec.Networks
+
+		for _, iface := range interfaces {
+			bindingType, bindingName := getBinding(iface)
+			networkName, matchFound := getNetworkName(iface.Name, networks)
+
+			if !matchFound {
+				continue
+			}
+
+			results = append(results, operatormetrics.CollectorResult{
+				Metric: vmVnicInfo,
+				Labels: []string{
+					vm.Name,
+					vm.Namespace,
+					iface.Name,
+					bindingType,
+					networkName,
+					bindingName,
+				},
+				Value: 1.0,
+			})
+		}
+	}
+
+	return results
+}
+
+func getBinding(iface k6tv1.Interface) (bindingType, bindingName string) {
+	switch {
+	case iface.Masquerade != nil:
+		bindingType = "core"
+		bindingName = "masquerade"
+	case iface.Bridge != nil:
+		bindingType = "core"
+		bindingName = "bridge"
+	case iface.SRIOV != nil:
+		bindingType = "core"
+		bindingName = "sriov"
+	case iface.Binding != nil:
+		bindingType = "plugin"
+		bindingName = iface.Binding.Name
+	}
+
+	return bindingType, bindingName
+}
+
+func getNetworkName(ifaceName string, networks []k6tv1.Network) (string, bool) {
+	if net := LookupNetworkByName(networks, ifaceName); net != nil {
+		if net.Pod != nil {
+			return "pod networking", true
+		} else if net.Multus != nil {
+			return net.Multus.NetworkName, true
+		}
+	}
+	return "", false
+}
+
+func LookupNetworkByName(networks []k6tv1.Network, name string) *k6tv1.Network {
+	for _, net := range networks {
+		if net.Name == name {
+			return &net
 		}
 	}
 	return nil
