@@ -177,7 +177,7 @@ var _ = Describe("VM Stats Collector", func() {
 
 				Expect(cr.GetLabelValue("machine_type")).To(Equal(vms[i].Spec.Template.Spec.Domain.Machine.Type))
 
-				Expect(cr.GetLabelValue("status")).To(Equal("CrashLoopBackOff"))
+				Expect(cr.GetLabelValue("status")).To(Equal("crashloopbackoff"))
 				Expect(cr.GetLabelValue("status_group")).To(Equal("error"))
 			}
 		})
@@ -191,10 +191,7 @@ var _ = Describe("VM Stats Collector", func() {
 				}
 			}
 
-			vms := []*k6tv1.VirtualMachine{{
-				ObjectMeta: metav1.ObjectMeta{Namespace: "test-ns"},
-				Spec:       k6tv1.VirtualMachineSpec{Instancetype: instanceType},
-			}}
+			vms := []*k6tv1.VirtualMachine{{Spec: k6tv1.VirtualMachineSpec{Instancetype: instanceType}}}
 			crs := CollectVMsInfo(vms)
 			Expect(crs).To(HaveLen(1), "Expected 1 metric")
 
@@ -211,10 +208,6 @@ var _ = Describe("VM Stats Collector", func() {
 			Entry("with no cluster instance type expect <none>", "VirtualMachineClusterInstancetype", "", "<none>"),
 			Entry("with managed cluster instance type expect its name", "VirtualMachineClusterInstancetype", "ci-managed", "ci-managed"),
 			Entry("with custom cluster instance type expect <other>", "VirtualMachineClusterInstancetype", "ci-unmanaged", "<other>"),
-			Entry("with an instance type which no longer exists expect <other>", "VirtualMachineInstancetype", "i-gone", "<other>"),
-			Entry("with a cluster instance type which no longer exists expect <other>", "VirtualMachineClusterInstancetype", "ci-gone", "<other>"),
-			Entry("with lowercase instance type expect the same behavior", "virtualmachineinstancetype", "i-managed", "i-managed"),
-			Entry("with lowercase cluster instance type expect the same behavior", "virtualmachineclusterinstancetype", "ci-managed", "ci-managed"),
 		)
 
 		DescribeTable("should show preference value correctly", func(preferenceAnnotationKey string, preferenceName string, expected string) {
@@ -226,10 +219,7 @@ var _ = Describe("VM Stats Collector", func() {
 				}
 			}
 
-			vms := []*k6tv1.VirtualMachine{{
-				ObjectMeta: metav1.ObjectMeta{Namespace: "test-ns"},
-				Spec:       k6tv1.VirtualMachineSpec{Preference: preference},
-			}}
+			vms := []*k6tv1.VirtualMachine{{Spec: k6tv1.VirtualMachineSpec{Preference: preference}}}
 			crs := CollectVMsInfo(vms)
 			Expect(crs).To(HaveLen(1), "Expected 1 metric")
 
@@ -246,10 +236,6 @@ var _ = Describe("VM Stats Collector", func() {
 			Entry("with no cluster preference expect <none>", "VirtualMachineClusterPreference", "", "<none>"),
 			Entry("with managed cluster preference expect its name", "VirtualMachineClusterPreference", "cp-managed", "cp-managed"),
 			Entry("with custom cluster preference expect <other>", "VirtualMachineClusterPreference", "cp-unmanaged", "<other>"),
-			Entry("with an preference which no longer exists expect <other>", "VirtualMachinePreference", "p-gone", "<other>"),
-			Entry("with a cluster preference which no longer exists expect <other>", "VirtualMachineClusterPreference", "cp-gone", "<other>"),
-			Entry("with lowercase preference expect the same behavior", "virtualmachinepreference", "p-managed", "p-managed"),
-			Entry("with lowercase cluster preference expect the same behavior", "virtualmachineclusterpreference", "cp-managed", "cp-managed"),
 		)
 	})
 
@@ -512,7 +498,7 @@ var _ = Describe("VM Stats Collector", func() {
 				},
 			}
 
-			results := CollectDiskAllocatedSize(vm)
+			results := CollectDiskAllocatedSize([]*k6tv1.VirtualMachine{vm})
 
 			Expect(results).ToNot(BeEmpty())
 			Expect(results[0].Metric.GetOpts().Name).To(Equal("kubevirt_vm_disk_allocated_size_bytes"))
@@ -550,7 +536,7 @@ var _ = Describe("VM Stats Collector", func() {
 				},
 			}
 
-			results := CollectDiskAllocatedSize(vm)
+			results := CollectDiskAllocatedSize([]*k6tv1.VirtualMachine{vm})
 
 			Expect(results).ToNot(BeEmpty())
 			Expect(results[0].Metric.GetOpts().Name).To(Equal("kubevirt_vm_disk_allocated_size_bytes"))
@@ -602,7 +588,7 @@ var _ = Describe("VM Stats Collector", func() {
 				},
 			}
 
-			results := CollectDiskAllocatedSize(vm)
+			results := CollectDiskAllocatedSize([]*k6tv1.VirtualMachine{vm})
 
 			Expect(results).ToNot(BeEmpty())
 			Expect(results[0].Metric.GetOpts().Name).To(Equal("kubevirt_vm_disk_allocated_size_bytes"))
@@ -610,6 +596,177 @@ var _ = Describe("VM Stats Collector", func() {
 			Expect(results[0].Labels).To(Equal([]string{"test-vm-dv", "default", "test-dv-pvc", "Filesystem", "datavolumedisk"}))
 		})
 	})
+
+	Context("VM creation time metric collection", func() {
+		It("should collect VM creation time correctly", func() {
+			testTime := time.Now()
+
+			vm := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace:         "test-ns",
+					Name:              "test-vm",
+					CreationTimestamp: metav1.NewTime(testTime),
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{},
+				},
+			}
+
+			results := collectVMCreationTimestamp([]*k6tv1.VirtualMachine{vm})
+
+			Expect(results).ToNot(BeEmpty())
+			Expect(results[0].Metric.GetOpts().Name).To(Equal("kubevirt_vm_create_date_timestamp_seconds"))
+			Expect(results[0].Value).To(Equal(float64(testTime.Unix())))
+			Expect(results[0].Labels).To(Equal([]string{"test-vm", "test-ns"}))
+		})
+
+		It("should collect correct creation times for multiple VMs", func() {
+			vm1 := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace:         "test-ns",
+					Name:              "test-vm1",
+					CreationTimestamp: metav1.NewTime(time.Now().Add(-time.Hour)),
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{},
+				},
+			}
+			vm2 := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace:         "test-ns",
+					Name:              "test-vm2",
+					CreationTimestamp: metav1.NewTime(time.Now().Add(-2 * time.Hour)),
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{},
+				},
+			}
+
+			vms := []*k6tv1.VirtualMachine{vm1, vm2}
+			results := collectVMCreationTimestamp(vms)
+
+			Expect(results).To(HaveLen(2))
+			Expect(results[0].Value).To(Equal(float64(vm1.CreationTimestamp.Unix())))
+			Expect(results[1].Value).To(Equal(float64(vm2.CreationTimestamp.Unix())))
+		})
+
+		It("metric should not exists if the VM creation timestamp is zero", func() {
+			vm := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace:         "test-ns",
+					Name:              "test-vm-zero-time",
+					CreationTimestamp: metav1.Time{},
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{},
+				},
+			}
+
+			results := collectVMCreationTimestamp([]*k6tv1.VirtualMachine{vm})
+
+			Expect(results).To(BeEmpty(), "kubevirt_vm_create_date_timestamp_seconds should not be collected for VMs with zero creation timestamp")
+		})
+	})
+
+	Context("VM vNIC info", func() {
+		It("should collect metrics for vNICs with various binding types, including PluginBinding", func() {
+			vm := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "test-ns",
+					Name:      "test-vm",
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{
+						Spec: k6tv1.VirtualMachineInstanceSpec{
+							Domain: k6tv1.DomainSpec{
+								Devices: k6tv1.Devices{
+									Interfaces: []k6tv1.Interface{
+										{
+											Name: "iface1",
+											InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+												Bridge: &k6tv1.InterfaceBridge{},
+											},
+										},
+										{
+											Name: "iface2",
+											InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+												Masquerade: &k6tv1.InterfaceMasquerade{},
+											},
+										},
+										{
+											Name: "iface3",
+											InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+												SRIOV: &k6tv1.InterfaceSRIOV{},
+											},
+										},
+										{
+											Name:    "iface4",
+											Binding: &k6tv1.PluginBinding{Name: "custom-plugin"},
+										},
+									},
+								},
+							},
+							Networks: []k6tv1.Network{
+								{
+									Name:          "iface1",
+									NetworkSource: k6tv1.NetworkSource{Pod: &k6tv1.PodNetwork{}},
+								},
+								{
+									Name:          "iface2",
+									NetworkSource: k6tv1.NetworkSource{Pod: &k6tv1.PodNetwork{}},
+								},
+								{
+									Name:          "iface3",
+									NetworkSource: k6tv1.NetworkSource{Multus: &k6tv1.MultusNetwork{NetworkName: "multus-net"}},
+								},
+								{
+									Name:          "iface4",
+									NetworkSource: k6tv1.NetworkSource{Multus: &k6tv1.MultusNetwork{NetworkName: "custom-net"}},
+								},
+							},
+						},
+					},
+				},
+			}
+
+			metrics := CollectVmsVnicInfo([]*k6tv1.VirtualMachine{vm})
+			Expect(metrics).To(HaveLen(4), "Expected metrics for all vNICs")
+
+			Expect(metrics[0].Labels).To(Equal([]string{"test-vm", "test-ns", "iface1", "core", "pod networking", "bridge"}))
+			Expect(metrics[1].Labels).To(Equal([]string{"test-vm", "test-ns", "iface2", "core", "pod networking", "masquerade"}))
+			Expect(metrics[2].Labels).To(Equal([]string{"test-vm", "test-ns", "iface3", "core", "multus-net", "sriov"}))
+			Expect(metrics[3].Labels).To(Equal([]string{"test-vm", "test-ns", "iface4", "plugin", "custom-net", "custom-plugin"}))
+		})
+		It("should not collect kubevirt_vm_vnic_info metric if no network defined", func() {
+			vm := &k6tv1.VirtualMachine{
+				ObjectMeta: metav1.ObjectMeta{
+					Namespace: "test-ns",
+					Name:      "test-vm",
+				},
+				Spec: k6tv1.VirtualMachineSpec{
+					Template: &k6tv1.VirtualMachineInstanceTemplateSpec{
+						Spec: k6tv1.VirtualMachineInstanceSpec{
+							Domain: k6tv1.DomainSpec{
+								Devices: k6tv1.Devices{
+									Interfaces: []k6tv1.Interface{
+										{
+											Name: "iface1",
+											InterfaceBindingMethod: k6tv1.InterfaceBindingMethod{
+												Bridge: &k6tv1.InterfaceBridge{},
+											},
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			}
+
+			metrics := CollectVmsVnicInfo([]*k6tv1.VirtualMachine{vm})
+			Expect(metrics).To(BeEmpty())
+		})
+	})
 })
 
 func expectDefaultCPUResourceRequests(crs []operatormetrics.CollectorResult) {
