@@ -24,33 +24,30 @@ import (
 	"fmt"
 	"strings"
 
+	batchv1 "k8s.io/api/batch/v1"
+	v1 "k8s.io/api/core/v1"
+	storagev1 "k8s.io/api/storage/v1"
+	k8serrors "k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/api/resource"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/client-go/tools/cache"
 
-	"kubevirt.io/kubevirt/pkg/apimachinery/patch"
-
+	corev1 "kubevirt.io/api/core/v1"
+	"kubevirt.io/client-go/kubecli"
 	"kubevirt.io/client-go/log"
 	cdiv1 "kubevirt.io/containerized-data-importer-api/pkg/apis/core/v1beta1"
 
-	"k8s.io/client-go/tools/cache"
-
-	"kubevirt.io/kubevirt/pkg/tpm"
+	"kubevirt.io/kubevirt/pkg/apimachinery/patch"
+	"kubevirt.io/kubevirt/pkg/controller"
+	"kubevirt.io/kubevirt/pkg/pointer"
+	"kubevirt.io/kubevirt/pkg/util"
 	virtconfig "kubevirt.io/kubevirt/pkg/virt-config"
-
-	v1 "k8s.io/api/core/v1"
-	storagev1 "k8s.io/api/storage/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	corev1 "kubevirt.io/api/core/v1"
-	"kubevirt.io/client-go/kubecli"
 )
 
 const (
 	PVCPrefix = "persistent-state-for"
 	PVCSize   = "10Mi"
-
-	// LabelApplyStorageProfile is a label used by the CDI mutating webhook
-	// to modify the PVC according to the storage profile.
-	LabelApplyStorageProfile = "cdi.kubevirt.io/applyStorageProfile"
 )
 
 func basePVC(vmi *corev1.VirtualMachineInstance) string {
@@ -66,6 +63,9 @@ func PVCForVMI(pvcStore cache.Store, vmi *corev1.VirtualMachineInstance) *v1.Per
 		if pvc.Namespace != vmi.Namespace {
 			continue
 		}
+		if pvc.DeletionTimestamp != nil {
+			continue
+		}
 		vmName, found := pvc.Labels[PVCPrefix]
 		if found && vmName == vmi.Name {
 			return pvc
@@ -97,7 +97,8 @@ func pvcForMigrationTargetFromStore(pvcStore cache.Store, migration *corev1.Virt
 
 func PVCForMigrationTarget(pvcStore cache.Store, migration *corev1.VirtualMachineInstanceMigration) *v1.PersistentVolumeClaim {
 	if migration.Status.MigrationState != nil && migration.Status.MigrationState.TargetPersistentStatePVCName != "" {
-		obj, exists, err := pvcStore.GetByKey(migration.Namespace + "/" + migration.Status.MigrationState.TargetPersistentStatePVCName)
+		key := controller.NamespacedKey(migration.Namespace, migration.Status.MigrationState.TargetPersistentStatePVCName)
+		obj, exists, err := pvcStore.GetByKey(key)
 		if err != nil || !exists {
 			return nil
 		}
@@ -107,6 +108,139 @@ func PVCForMigrationTarget(pvcStore cache.Store, migration *corev1.VirtualMachin
 	return pvcForMigrationTargetFromStore(pvcStore, migration)
 }
 
+func RecoverFromBrokenMigration(client kubecli.KubevirtClient, migration *corev1.VirtualMachineInstanceMigration, pvcStore cache.Store, vmi *corev1.VirtualMachineInstance, launcherImage string) error {
+	if migration.Status.MigrationState == nil ||
+		migration.Status.MigrationState.TargetPersistentStatePVCName == migration.Status.MigrationState.SourcePersistentStatePVCName {
+		// The migration either didn't actually start, or the backend storage is RWX.
+		// In both cases we consider the migration as failed.
+		migration.Status.Phase = corev1.MigrationFailed
+		return nil
+	}
+
+	// An interrupted migration exists. Using a job to check if the source PVC contains /meta/migrated,
+	// which would indicate that the libvirt migration finished.
+	// A JobComplete condition indicates the file is present, the migration was successful and the target PVC prevails
+	// A JobFailed condition indicated the file is absent, the migration didn't finish and the source PVC prevails
+
+	jobName := "recover-" + migration.Name
+
+	job, err := client.BatchV1().Jobs(vmi.Namespace).Get(context.Background(), jobName, metav1.GetOptions{})
+	if err != nil {
+		if !k8serrors.IsNotFound(err) {
+			return err
+		}
+		job = buildRecoveryJob(jobName, launcherImage, migration)
+		job, err = client.BatchV1().Jobs(vmi.Namespace).Create(context.Background(), job, metav1.CreateOptions{})
+		if err != nil {
+			return err
+		}
+		// The job was just created, return an error to be re-enqueued to check on the job
+		return fmt.Errorf("a migration recovery had to be initiated")
+	}
+
+	for _, c := range job.Status.Conditions {
+		switch c.Type {
+		case batchv1.JobComplete:
+			if c.Status == v1.ConditionTrue {
+				err = MigrationHandoff(client, pvcStore, migration)
+				if err == nil {
+					migration.Status.Phase = corev1.MigrationSucceeded
+				}
+				return err
+			}
+		case batchv1.JobFailed:
+			if c.Status == v1.ConditionTrue {
+				if c.Reason == batchv1.JobReasonPodFailurePolicy {
+					// The job ran properly but didn't find /meta/migrated, meaning the migration failed
+					err = MigrationAbort(client, migration)
+					if err == nil {
+						migration.Status.Phase = corev1.MigrationFailed
+					}
+					return err
+				} else {
+					// The job failed to run properly. Deleting it to retry asap.
+					// Ignoring the deletion error because the job may already be gone, or will get auto-removed anyway.
+					_ = client.BatchV1().Jobs(job.Namespace).Delete(context.Background(), job.Name, metav1.DeleteOptions{
+						PropagationPolicy: pointer.P(metav1.DeletePropagationBackground),
+					})
+					return fmt.Errorf(c.Message)
+				}
+			}
+		default:
+			break
+		}
+	}
+
+	return fmt.Errorf("migration recovery job still running")
+}
+
+func buildRecoveryJob(jobName, launcherImage string, migration *corev1.VirtualMachineInstanceMigration) *batchv1.Job {
+	return &batchv1.Job{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: jobName,
+			OwnerReferences: []metav1.OwnerReference{
+				*metav1.NewControllerRef(migration, corev1.VirtualMachineInstanceMigrationGroupVersionKind),
+			},
+		},
+		Spec: batchv1.JobSpec{
+			ActiveDeadlineSeconds:   pointer.P(int64(30)),
+			BackoffLimit:            pointer.P(int32(0)),
+			TTLSecondsAfterFinished: pointer.P(int32(30)),
+			PodFailurePolicy: &batchv1.PodFailurePolicy{
+				Rules: []batchv1.PodFailurePolicyRule{{
+					Action: batchv1.PodFailurePolicyActionFailJob,
+					OnExitCodes: &batchv1.PodFailurePolicyOnExitCodesRequirement{
+						ContainerName: pointer.P("container"),
+						Operator:      "In",
+						Values:        []int32{42},
+					},
+				}},
+			},
+			Template: v1.PodTemplateSpec{
+				ObjectMeta: metav1.ObjectMeta{
+					GenerateName: jobName + "-",
+				},
+				Spec: v1.PodSpec{
+					RestartPolicy: v1.RestartPolicyNever,
+					SecurityContext: &v1.PodSecurityContext{
+						RunAsNonRoot: pointer.P(true),
+						RunAsUser:    pointer.P(int64(util.NonRootUID)),
+						RunAsGroup:   pointer.P(int64(util.NonRootUID)),
+						FSGroup:      pointer.P(int64(util.NonRootUID)),
+						SeccompProfile: &v1.SeccompProfile{
+							Type: v1.SeccompProfileTypeRuntimeDefault,
+						},
+					},
+					Containers: []v1.Container{{
+						Name: "container",
+						SecurityContext: &v1.SecurityContext{
+							AllowPrivilegeEscalation: pointer.P(false),
+							Capabilities:             &v1.Capabilities{Drop: []v1.Capability{"ALL"}},
+						},
+						Image:   launcherImage,
+						Command: []string{"bash"},
+						Args:    []string{"-c", "ls /meta/migrated || exit 42"},
+						VolumeMounts: []v1.VolumeMount{{
+							Name:      "backend-storage",
+							MountPath: "/meta",
+							SubPath:   "meta",
+						}},
+					}},
+					Volumes: []v1.Volume{{
+						Name: "backend-storage",
+						VolumeSource: v1.VolumeSource{
+							PersistentVolumeClaim: &v1.PersistentVolumeClaimVolumeSource{
+								ClaimName: migration.Status.MigrationState.SourcePersistentStatePVCName,
+							},
+						},
+					}},
+				},
+			},
+		},
+	}
+
+}
+
 func (bs *BackendStorage) labelLegacyPVC(pvc *v1.PersistentVolumeClaim, name string) {
 	labelPatch := patch.New()
 	if len(pvc.Labels) == 0 {
@@ -133,6 +267,12 @@ func CurrentPVCName(vmi *corev1.VirtualMachineInstance) string {
 	return ""
 }
 
+func HasPersistentTPMDevice(vmiSpec *corev1.VirtualMachineInstanceSpec) bool {
+	return vmiSpec.Domain.Devices.TPM != nil &&
+		vmiSpec.Domain.Devices.TPM.Persistent != nil &&
+		*vmiSpec.Domain.Devices.TPM.Persistent
+}
+
 func HasPersistentEFI(vmiSpec *corev1.VirtualMachineInstanceSpec) bool {
 	return vmiSpec.Domain.Firmware != nil &&
 		vmiSpec.Domain.Firmware.Bootloader != nil &&
@@ -142,16 +282,18 @@ func HasPersistentEFI(vmiSpec *corev1.VirtualMachineInstanceSpec) bool {
 }
 
 func IsBackendStorageNeededForVMI(vmiSpec *corev1.VirtualMachineInstanceSpec) bool {
-	return tpm.HasPersistentDevice(vmiSpec) || HasPersistentEFI(vmiSpec)
+	return HasPersistentTPMDevice(vmiSpec) || HasPersistentEFI(vmiSpec)
 }
 
 func IsBackendStorageNeededForVM(vm *corev1.VirtualMachine) bool {
 	if vm.Spec.Template == nil {
 		return false
 	}
-	return tpm.HasPersistentDevice(&vm.Spec.Template.Spec) || HasPersistentEFI(&vm.Spec.Template.Spec)
+	return HasPersistentTPMDevice(&vm.Spec.Template.Spec) || HasPersistentEFI(&vm.Spec.Template.Spec)
 }
 
+// MigrationHandoff runs at the end of a successful live migration.
+// It labels the target backend-storage PVC as current for the VM and deletes the source backend-storage PVC.
 func MigrationHandoff(client kubecli.KubevirtClient, pvcStore cache.Store, migration *corev1.VirtualMachineInstanceMigration) error {
 	if migration == nil || migration.Status.MigrationState == nil ||
 		migration.Status.MigrationState.SourcePersistentStatePVCName == "" ||
@@ -200,13 +342,15 @@ func MigrationHandoff(client kubecli.KubevirtClient, pvcStore cache.Store, migra
 	}
 
 	err := client.CoreV1().PersistentVolumeClaims(migration.Namespace).Delete(context.Background(), sourcePVC, metav1.DeleteOptions{})
-	if err != nil {
+	if err != nil && !k8serrors.IsNotFound(err) {
 		return fmt.Errorf("failed to delete PVC: %v", err)
 	}
 
 	return nil
 }
 
+// MigrationAbort runs at the end of a failed live migration.
+// It just removes the target backend-storage PVC.
 func MigrationAbort(client kubecli.KubevirtClient, migration *corev1.VirtualMachineInstanceMigration) error {
 	if migration == nil || migration.Status.MigrationState == nil ||
 		migration.Status.MigrationState.TargetPersistentStatePVCName == "" {
@@ -222,7 +366,7 @@ func MigrationAbort(client kubecli.KubevirtClient, migration *corev1.VirtualMach
 	}
 
 	err := client.CoreV1().PersistentVolumeClaims(migration.Namespace).Delete(context.Background(), targetPVC, metav1.DeleteOptions{})
-	if err != nil {
+	if err != nil && !k8serrors.IsNotFound(err) {
 		return fmt.Errorf("failed to delete PVC: %v", err)
 	}
 
@@ -360,15 +504,6 @@ func (bs *BackendStorage) createPVC(vmi *corev1.VirtualMachineInstance, labels m
 			*metav1.NewControllerRef(vmi, corev1.VirtualMachineInstanceGroupVersionKind),
 		}
 	}
-
-	// Adding this label to allow the PVC to be processed by the CDI WebhookPvcRendering mutating webhook,
-	// which must be enabled in the CDI CR via feature gate.
-	// This mutating webhook processes the PVC based on its associated StorageProfile.
-	// For example, a profile can define a minimum supported volume size via the annotation:
-	// cdi.kubevirt.io/minimumSupportedPvcSize: 4Gi
-	// This helps avoid issues with provisioners that reject the hardcoded 10Mi PVC size used here.
-	labels[LabelApplyStorageProfile] = "true"
-
 	pvc := &v1.PersistentVolumeClaim{
 		ObjectMeta: metav1.ObjectMeta{
 			GenerateName:    basePVC(vmi) + "-",
@@ -426,7 +561,7 @@ func (bs *BackendStorage) IsPVCReady(vmi *corev1.VirtualMachineInstance, pvcName
 		return true, nil
 	}
 
-	obj, exists, err := bs.pvcStore.GetByKey(vmi.Namespace + "/" + pvcName)
+	obj, exists, err := bs.pvcStore.GetByKey(controller.NamespacedKey(vmi.Namespace, pvcName))
 	if err != nil {
 		return false, err
 	}
