@@ -61,7 +61,6 @@ import (
 	storageutils "kubevirt.io/kubevirt/pkg/storage/utils"
 	kutil "kubevirt.io/kubevirt/pkg/util"
 	virtconfig "kubevirt.io/kubevirt/pkg/virt-config"
-	"kubevirt.io/kubevirt/pkg/virt-controller/services"
 	watchutil "kubevirt.io/kubevirt/pkg/virt-controller/watch/util"
 	"kubevirt.io/kubevirt/pkg/virt-operator/resource/apply"
 	"kubevirt.io/kubevirt/pkg/virt-operator/resource/generate/components"
@@ -112,8 +111,6 @@ const (
 
 	kvm = 107
 
-	// secretTokenLength is the lenght of the randomly generated token
-	secretTokenLength = 20
 	// secretTokenKey is the entry used to store the token in the virtualMachineExport secret
 	secretTokenKey = "token"
 
@@ -178,11 +175,15 @@ func (sv *sourceVolumes) isSourceAvailable() bool {
 	return !sv.inUse && sv.isPopulated
 }
 
+type manifestRenderer interface {
+	RenderExporterManifest(vmExport *exportv1.VirtualMachineExport, namePrefix string) *corev1.Pod
+}
+
 // VMExportController is resonsible for exporting VMs
 type VMExportController struct {
 	Client kubecli.KubevirtClient
 
-	TemplateService services.TemplateService
+	ManifestRenderer manifestRenderer
 
 	VMExportInformer            cache.SharedIndexInformer
 	PVCInformer                 cache.SharedIndexInformer
@@ -211,7 +212,7 @@ type VMExportController struct {
 
 	KubevirtNamespace string
 
-	vmExportQueue workqueue.RateLimitingInterface
+	vmExportQueue workqueue.TypedRateLimitingInterface[string]
 
 	caCertManager *bootstrap.FileCertificateManager
 
@@ -262,7 +263,10 @@ func (ctrl *VMExportController) Init() error {
 	if err != nil {
 		return err
 	}
-	ctrl.vmExportQueue = workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "virt-controller-export-vmexport")
+	ctrl.vmExportQueue = workqueue.NewTypedRateLimitingQueueWithConfig[string](
+		workqueue.DefaultTypedControllerRateLimiter[string](),
+		workqueue.TypedRateLimitingQueueConfig[string]{Name: "virt-controller-export-vmexport"},
+	)
 
 	_, err = ctrl.VMExportInformer.AddEventHandler(
 		cache.ResourceEventHandlerFuncs{
@@ -500,6 +504,12 @@ func (ctrl *VMExportController) updateVMExport(vmExport *exportv1.VirtualMachine
 	if vmExport.DeletionTimestamp != nil {
 		return 0, nil
 	}
+	if vmExport.Labels == nil {
+		vmExport.Labels = make(map[string]string)
+	}
+	if vmExport.Annotations == nil {
+		vmExport.Annotations = make(map[string]string)
+	}
 
 	service, err := ctrl.getOrCreateExportService(vmExport)
 	if err != nil {
@@ -721,7 +731,7 @@ func (ctrl *VMExportController) handleVMExportToken(vmExport *exportv1.VirtualMa
 		vmExport.Status.TokenSecretRef = &generatedSecretName
 	}
 
-	token, err := kutil.GenerateSecureRandomString(secretTokenLength)
+	token, err := kutil.GenerateVMExportToken()
 	if err != nil {
 		return err
 	}
@@ -778,15 +788,6 @@ func (ctrl *VMExportController) getExportPodName(vmExport *exportv1.VirtualMachi
 	return naming.GetName(exportPrefix, vmExport.Name, validation.DNS1035LabelMaxLength)
 }
 
-func (ctrl *VMExportController) getExportPodVolumeName(pvc *corev1.PersistentVolumeClaim) string {
-	pvcName := strings.ReplaceAll(pvc.Name, ".", "-")
-	// Using the formatted PVC name if it's under the max length.
-	if len(pvcName) <= validation.DNS1035LabelMaxLength {
-		return pvcName
-	}
-	return naming.GetName(exportPrefix, pvcName, validation.DNS1035LabelMaxLength)
-}
-
 // getExportLabelValue will return the virtual machine's name if it is under the
 // DNS1035-specified max length, or a normalized name otherwise.
 func (ctrl *VMExportController) getExportLabelValue(vmExport *exportv1.VirtualMachineExport) string {
@@ -816,6 +817,11 @@ func (ctrl *VMExportController) getOrCreateExportService(vmExport *exportv1.Virt
 }
 
 func (ctrl *VMExportController) createServiceManifest(vmExport *exportv1.VirtualMachineExport) *corev1.Service {
+	labels := map[string]string{virtv1.AppLabel: exportv1.App}
+	for key, value := range vmExport.Labels {
+		labels[key] = value
+	}
+
 	service := &corev1.Service{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      ctrl.getExportServiceName(vmExport),
@@ -827,9 +833,8 @@ func (ctrl *VMExportController) createServiceManifest(vmExport *exportv1.Virtual
 					Kind:    exportGVK.Kind,
 				}),
 			},
-			Labels: map[string]string{
-				virtv1.AppLabel: exportv1.App,
-			},
+			Labels:      labels,
+			Annotations: vmExport.Annotations,
 		},
 		Spec: corev1.ServiceSpec{
 			Ports: []corev1.ServicePort{
@@ -900,9 +905,15 @@ func (ctrl *VMExportController) createExporterPodManifest(vmExport *exportv1.Vir
 	}
 
 	deadline := certParams.Duration - certParams.RenewBefore
-	podManifest := ctrl.TemplateService.RenderExporterManifest(vmExport, exportPrefix)
+	podManifest := ctrl.ManifestRenderer.RenderExporterManifest(vmExport, exportPrefix)
 	podManifest.Labels = map[string]string{exportServiceLabel: ctrl.getExportLabelValue(vmExport)}
+	for key, value := range vmExport.Labels {
+		podManifest.Labels[key] = value
+	}
 	podManifest.Annotations = map[string]string{annCertParams: scp}
+	for key, value := range vmExport.Annotations {
+		podManifest.Annotations[key] = value
+	}
 	podManifest.Spec.SecurityContext = &corev1.PodSecurityContext{
 		RunAsNonRoot:   pointer.P(true),
 		FSGroup:        pointer.P(int64(kvm)),
@@ -910,7 +921,7 @@ func (ctrl *VMExportController) createExporterPodManifest(vmExport *exportv1.Vir
 	}
 	for i, pvc := range pvcs {
 		var mountPoint string
-		volumeName := ctrl.getExportPodVolumeName(pvc)
+		volumeName := strings.ReplaceAll(pvc.Name, ".", "-")
 		if types.IsPVCBlock(pvc.Spec.VolumeMode) {
 			mountPoint = fmt.Sprintf("%s/%s", blockVolumeMountPath, volumeName)
 			podManifest.Spec.Containers[0].VolumeDevices = append(podManifest.Spec.Containers[0].VolumeDevices, corev1.VolumeDevice{
@@ -1081,7 +1092,6 @@ func (ctrl *VMExportController) createDataManifestConfigMap(vmExport *exportv1.V
 	if err != nil {
 		return nil, err
 	}
-
 	for _, datavolume := range datavolumes {
 		if datavolume != nil {
 			dvBytes, err := json.Marshal(datavolume)
@@ -1457,7 +1467,6 @@ func (ctrl *VMExportController) generateVMDefinitionFromVm(vm *virtv1.VirtualMac
 	if err != nil {
 		return nil, err
 	}
-
 	vmBytes, err := json.Marshal(expandedVm)
 	if err != nil {
 		return nil, err
@@ -1496,26 +1505,26 @@ func (ctrl *VMExportController) generateDataVolumesFromVm(vm *virtv1.VirtualMach
 
 func (ctrl *VMExportController) createExportHttpDvFromPVC(namespace, name string) *cdiv1.DataVolume {
 	pvc := ctrl.getPVCsFromName(namespace, name)
-	if pvc != nil {
-		pvc.Spec.VolumeName = ""
-		pvc.Spec.StorageClassName = nil
-		// Don't copy datasources, will be populated by CDI with the datavolume
-		pvc.Spec.DataSource = nil
-		pvc.Spec.DataSourceRef = nil
-		return &cdiv1.DataVolume{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      name,
-				Namespace: namespace,
+	if pvc == nil {
+		return nil
+	}
+
+	return &cdiv1.DataVolume{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: namespace,
+		},
+		Spec: cdiv1.DataVolumeSpec{
+			Source: &cdiv1.DataVolumeSource{
+				HTTP: &cdiv1.DataVolumeSourceHTTP{
+					URL: "",
+				},
 			},
-			Spec: cdiv1.DataVolumeSpec{
-				Source: &cdiv1.DataVolumeSource{
-					HTTP: &cdiv1.DataVolumeSourceHTTP{
-						URL: "",
-					},
-				},
-				PVC: &pvc.Spec,
+			Storage: &cdiv1.StorageSpec{
+				AccessModes: pvc.Spec.AccessModes,
+				VolumeMode:  pvc.Spec.VolumeMode,
+				Resources:   pvc.Spec.Resources,
 			},
-		}
+		},
 	}
-	return nil
 }
