@@ -28,9 +28,12 @@ import (
 	"strings"
 	"time"
 
+	"k8s.io/apimachinery/pkg/util/uuid"
+
 	"github.com/golang/mock/gomock"
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
+	"github.com/onsi/gomega/gstruct"
 
 	routev1 "github.com/openshift/api/route/v1"
 	appsv1 "k8s.io/api/apps/v1"
@@ -38,6 +41,7 @@ import (
 	networkingv1 "k8s.io/api/networking/v1"
 	extv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/runtime/schema"
@@ -72,9 +76,14 @@ import (
 )
 
 const (
-	testNamespace  = "default"
-	ingressSecret  = "ingress-secret"
-	currentVersion = "v1beta1"
+	testNamespace   = "default"
+	ingressSecret   = "ingress-secret"
+	currentVersion  = "v1beta1"
+	vmExportName    = "test"
+	labelKey        = "label-key"
+	labelValue      = "label-value"
+	annotationKey   = "annotation-key"
+	annotationValue = "annotation-value"
 )
 
 var (
@@ -125,7 +134,7 @@ var _ = Describe("Export controller", func() {
 		virtClient                  *kubecli.MockKubevirtClient
 		vmExportClient              *kubevirtfake.Clientset
 		fakeVolumeSnapshotProvider  *MockVolumeSnapshotProvider
-		mockVMExportQueue           *testutils.MockWorkQueue
+		mockVMExportQueue           *testutils.MockWorkQueue[string]
 		routeCache                  cache.Store
 		ingressCache                cache.Store
 		certDir                     string
@@ -237,7 +246,7 @@ var _ = Describe("Export controller", func() {
 			ServiceInformer:             serviceInformer,
 			DataVolumeInformer:          dvInformer,
 			KubevirtNamespace:           "kubevirt",
-			TemplateService:             services.NewTemplateService("a", 240, "b", "c", "d", "e", "f", "g", pvcInformer.GetStore(), virtClient, config, qemuGid, "h", rqInformer.GetStore(), nsInformer.GetStore()),
+			ManifestRenderer:            services.NewTemplateService("a", 240, "b", "c", "d", "e", "f", pvcInformer.GetStore(), virtClient, config, qemuGid, "g", rqInformer.GetStore(), nsInformer.GetStore()),
 			caCertManager:               bootstrap.NewFileCertificateManager(certFilePath, keyFilePath),
 			RouteCache:                  routeCache,
 			IngressCache:                ingressCache,
@@ -744,6 +753,10 @@ var _ = Describe("Export controller", func() {
 			service.Status.Conditions[0].Type = "test"
 			Expect(service.GetName()).To(Equal(controller.getExportServiceName(testVMExport)))
 			Expect(service.GetNamespace()).To(Equal(testNamespace))
+			Expect(service.Labels).To(And(
+				HaveKeyWithValue(virtv1.AppLabel, "virt-exporter"),
+				HaveKeyWithValue(labelKey, labelValue)))
+			Expect(service.Annotations).To(HaveKeyWithValue(annotationKey, annotationValue))
 			return true, service, nil
 		})
 
@@ -915,7 +928,14 @@ var _ = Describe("Export controller", func() {
 			Name:       testPVC.Name,
 			DevicePath: fmt.Sprintf("%s/%s", blockVolumeMountPath, testPVC.Name),
 		}))
-		Expect(pod.Annotations[annCertParams]).To(Equal("{\"Duration\":7200000000000,\"RenewBefore\":3600000000000}"))
+		Expect(pod.Labels).To(And(
+			HaveKeyWithValue(exportServiceLabel, controller.getExportLabelValue(testVMExport)),
+			HaveKeyWithValue(labelKey, labelValue)))
+		Expect(pod.Annotations).To(And(
+			HaveKeyWithValue(annCertParams, fmt.Sprintf("{\"Duration\":%d,\"RenewBefore\":%d}",
+				metav1.Duration{Duration: 2 * time.Hour}.Nanoseconds(),
+				metav1.Duration{Duration: 1 * time.Hour}.Nanoseconds())),
+			HaveKeyWithValue(annotationKey, annotationValue)))
 		Expect(pod.Spec.Containers[0].Env).To(ContainElements(expectedPodEnvVars))
 		Expect(pod.Spec.Containers[0].Resources.Requests.Cpu()).ToNot(BeNil())
 		Expect(pod.Spec.Containers[0].Resources.Requests.Cpu().MilliValue()).To(Equal(int64(100)))
@@ -934,65 +954,6 @@ var _ = Describe("Export controller", func() {
 		Entry("Snapshot", populateVmExportVMSnapshot, controller.getPVCFromSourceVMSnapshot, 4),
 	)
 
-	DescribeTable("Volumemount names should be trimmed depending on the PVC name", func(pvcName string) {
-		testVMExport := createPVCVMExportWithName(pvcName)
-		testPVC := &k8sv1.PersistentVolumeClaim{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      pvcName,
-				Namespace: testNamespace,
-			},
-			Spec: k8sv1.PersistentVolumeClaimSpec{
-				VolumeMode: (*k8sv1.PersistentVolumeMode)(pointer.P(string(k8sv1.PersistentVolumeBlock))),
-			},
-		}
-		populateInitialVMExportStatus(testVMExport)
-		err := controller.handleVMExportToken(testVMExport, controller.getPVCFromSourceVMSnapshot)
-		Expect(testVMExport.Status.TokenSecretRef).ToNot(BeNil())
-		Expect(err).ToNot(HaveOccurred())
-		k8sClient.Fake.PrependReactor("create", "pods", func(action testing.Action) (handled bool, obj runtime.Object, err error) {
-			create, ok := action.(testing.CreateAction)
-			Expect(ok).To(BeTrue())
-			pod, ok := create.GetObject().(*k8sv1.Pod)
-			Expect(ok).To(BeTrue())
-			Expect(pod.GetName()).To(Equal(controller.getExportPodName(testVMExport)))
-			Expect(len(pod.GetName())).To(BeNumerically("<=", validation.DNS1035LabelMaxLength))
-			Expect(pod.GetNamespace()).To(Equal(testNamespace))
-			return true, pod, nil
-		})
-		var service *k8sv1.Service
-		k8sClient.Fake.PrependReactor("create", "services", func(action testing.Action) (handled bool, obj runtime.Object, err error) {
-			create, ok := action.(testing.CreateAction)
-			Expect(ok).To(BeTrue())
-			service, ok = create.GetObject().(*k8sv1.Service)
-			Expect(ok).To(BeTrue())
-			service.Status.Conditions = make([]metav1.Condition, 1)
-			service.Status.Conditions[0].Type = "test"
-			Expect(service.GetName()).To(Equal(controller.getExportServiceName(testVMExport)))
-			Expect(service.GetNamespace()).To(Equal(testNamespace))
-			return true, service, nil
-		})
-		service, err = controller.getOrCreateExportService(testVMExport)
-		Expect(err).ToNot(HaveOccurred())
-		pod, err := controller.createExporterPod(testVMExport, service, []*k8sv1.PersistentVolumeClaim{testPVC})
-		Expect(err).ToNot(HaveOccurred())
-		Expect(pod).ToNot(BeNil())
-		Expect(pod.Spec.Containers).To(HaveLen(1))
-		Expect(pod.Spec.Containers[0].VolumeDevices).To(HaveLen(1))
-		Expect(pod.Spec.Containers[0].VolumeDevices).To(ContainElement(k8sv1.VolumeDevice{
-			Name:       controller.getExportPodVolumeName(testPVC),
-			DevicePath: fmt.Sprintf("%s/%s", blockVolumeMountPath, controller.getExportPodVolumeName(testPVC)),
-		}))
-		if len(pvcName) > validation.DNS1035LabelMaxLength {
-			Expect(len(pod.Spec.Containers[0].VolumeDevices[0].Name)).To(BeNumerically("<", 63))
-		} else {
-			Expect(pod.Spec.Containers[0].VolumeDevices[0].Name).To(Equal(pvcName))
-		}
-	},
-		Entry("PVC name within limit", "pvc-name-within-limit"),
-		Entry("PVC name exceeding limit", strings.Repeat("a", validation.DNS1035LabelMaxLength+1)),
-		Entry("PVC name with same length as limit", strings.Repeat("a", validation.DNS1035LabelMaxLength)),
-	)
-
 	It("Should create a secret based on the vm export", func() {
 		cp := &CertParams{Duration: 24 * time.Hour, RenewBefore: 2 * time.Hour}
 		scp, err := serializeCertParams(cp)
@@ -1434,16 +1395,33 @@ var _ = Describe("Export controller", func() {
 		pvc := createPVC("pvc", string(cdiv1.DataVolumeKubeVirt))
 		pvc.Spec.DataSource = &k8sv1.TypedLocalObjectReference{}
 		pvc.Spec.DataSourceRef = &k8sv1.TypedObjectReference{}
+		pvc.Spec.StorageClassName = pointer.P("somesc")
+		pvcCopy := pvc.DeepCopy()
 		pvcInformer.GetStore().Add(pvc)
 		vm := createVMWithDVTemplateAndPVC()
 		dvs, err := controller.generateDataVolumesFromVm(vm)
 		Expect(err).ToNot(HaveOccurred())
 		Expect(dvs).To(HaveLen(1))
 		Expect(dvs[0]).ToNot(BeNil())
-		Expect(dvs[0].Name).To((Equal("pvc")))
-		Expect(dvs[0].Spec.PVC.DataSource).To(BeNil())
-		Expect(dvs[0].Spec.PVC.DataSourceRef).To(BeNil())
-		Expect(dvs[0].Spec.SourceRef).To(BeNil())
+		Expect(dvs[0].Name).To(Equal("pvc"))
+		Expect(dvs[0].Spec.Storage).To(
+			gstruct.PointTo(
+				gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+					"DataSource":       BeNil(),
+					"DataSourceRef":    BeNil(),
+					"StorageClassName": BeNil(),
+					"VolumeMode":       Equal(pvcCopy.Spec.VolumeMode),
+					"Resources":        Equal(pvcCopy.Spec.Resources),
+					"AccessModes":      HaveExactElements(pvcCopy.Spec.AccessModes),
+				}),
+			),
+		)
+		Expect(dvs[0].Spec).To(
+			gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+				"PVC":       BeNil(),
+				"SourceRef": BeNil(),
+			}),
+		)
 	})
 })
 
@@ -1590,21 +1568,28 @@ func writeCertsToDir(dir string) {
 	Expect(os.WriteFile(filepath.Join(dir, bootstrap.KeyBytesValue), key, 0777)).To(Succeed())
 }
 
+func createVMExportMeta(name string) metav1.ObjectMeta {
+	return metav1.ObjectMeta{
+		Name:              name,
+		Namespace:         testNamespace,
+		UID:               uuid.NewUUID(),
+		CreationTimestamp: metav1.Now(),
+		Labels:            map[string]string{labelKey: labelValue},
+		Annotations:       map[string]string{annotationKey: annotationValue},
+	}
+}
+
 func createPVCVMExport() *exportv1.VirtualMachineExport {
-	return createPVCVMExportWithName("test")
+	return createPVCVMExportWithName(vmExportName)
 }
 
 func createPVCVMExportLongName() *exportv1.VirtualMachineExport {
-	return createPVCVMExportWithName("test" + strings.Repeat("a", 63))
+	return createPVCVMExportWithName(vmExportName + strings.Repeat("a", 63))
 }
 
 func createPVCVMExportWithName(name string) *exportv1.VirtualMachineExport {
 	return &exportv1.VirtualMachineExport{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:              name,
-			Namespace:         testNamespace,
-			CreationTimestamp: metav1.Now(),
-		},
+		ObjectMeta: createVMExportMeta(name),
 		Spec: exportv1.VirtualMachineExportSpec{
 			Source: k8sv1.TypedLocalObjectReference{
 				APIGroup: &k8sv1.SchemeGroupVersion.Group,
@@ -1618,10 +1603,7 @@ func createPVCVMExportWithName(name string) *exportv1.VirtualMachineExport {
 
 func createPVCVMExportWithoutSecret() *exportv1.VirtualMachineExport {
 	return &exportv1.VirtualMachineExport{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "test-no-secret",
-			Namespace: testNamespace,
-		},
+		ObjectMeta: createVMExportMeta(vmExportName),
 		Spec: exportv1.VirtualMachineExportSpec{
 			Source: k8sv1.TypedLocalObjectReference{
 				APIGroup: &k8sv1.SchemeGroupVersion.Group,
@@ -1634,12 +1616,7 @@ func createPVCVMExportWithoutSecret() *exportv1.VirtualMachineExport {
 
 func createSnapshotVMExport() *exportv1.VirtualMachineExport {
 	return &exportv1.VirtualMachineExport{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:              "test",
-			Namespace:         testNamespace,
-			UID:               "11111-22222-33333",
-			CreationTimestamp: metav1.Now(),
-		},
+		ObjectMeta: createVMExportMeta(vmExportName),
 		Spec: exportv1.VirtualMachineExportSpec{
 			Source: k8sv1.TypedLocalObjectReference{
 				APIGroup: &snapshotv1.SchemeGroupVersion.Group,
@@ -1653,12 +1630,7 @@ func createSnapshotVMExport() *exportv1.VirtualMachineExport {
 
 func createVMVMExport() *exportv1.VirtualMachineExport {
 	return &exportv1.VirtualMachineExport{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:              "test",
-			Namespace:         testNamespace,
-			UID:               "44444-555555-666666",
-			CreationTimestamp: metav1.Now(),
-		},
+		ObjectMeta: createVMExportMeta(vmExportName),
 		Spec: exportv1.VirtualMachineExportSpec{
 			Source: k8sv1.TypedLocalObjectReference{
 				APIGroup: &virtv1.SchemeGroupVersion.Group,
@@ -1692,6 +1664,15 @@ func createPVC(name, contentType string) *k8sv1.PersistentVolumeClaim {
 				annContentType: contentType,
 			},
 		},
+		Spec: k8sv1.PersistentVolumeClaimSpec{
+			VolumeMode:  pointer.P(k8sv1.PersistentVolumeMode("testvolumemode")),
+			AccessModes: []k8sv1.PersistentVolumeAccessMode{k8sv1.PersistentVolumeAccessMode("testaccessmode")},
+			Resources: k8sv1.VolumeResourceRequirements{
+				Requests: k8sv1.ResourceList{
+					k8sv1.ResourceStorage: resource.MustParse("99Gi"),
+				},
+			},
+		},
 		Status: k8sv1.PersistentVolumeClaimStatus{
 			Phase: k8sv1.ClaimBound,
 		},
