@@ -72,10 +72,20 @@ const (
 
 	restoreErrorEvent = "VirtualMachineRestoreError"
 
+	restoreVMNotReadyEvent = "RestoreTargetNotReady"
+
 	restoreDataVolumeCreateErrorEvent = "RestoreDataVolumeCreateError"
 )
 
+var (
+	restoreGracePeriodExceededError = fmt.Sprintf("Restore target failed to be ready within %s", snapshotv1.DefaultGracePeriod)
+	restoreTargetNotReady           = "Restore target not ready"
+	restoredFailed                  = "Operation failed"
+	errorRestoreToExistingTarget    = "restore source and restore target are different but restore target already exists"
+)
+
 type restoreTarget interface {
+	Stop() error
 	Ready() (bool, error)
 	Reconcile() (bool, error)
 	Own(obj metav1.Object)
@@ -83,7 +93,9 @@ type restoreTarget interface {
 	UpdateRestoreInProgress() error
 	UpdateTarget(obj metav1.Object)
 	Exists() bool
+	UID() types.UID
 	VirtualMachine() *kubevirtv1.VirtualMachine
+	TargetRestored() bool
 }
 
 type vmRestoreTarget struct {
@@ -109,8 +121,17 @@ func restoreDVName(vmRestore *snapshotv1.VirtualMachineRestore, name string) str
 	return restorePVCName(vmRestore, name)
 }
 
+func vmRestoreFailed(vmRestore *snapshotv1.VirtualMachineRestore) bool {
+	return vmRestore.Status != nil &&
+		hasConditionType(vmRestore.Status.Conditions, snapshotv1.ConditionFailure)
+}
+
+func vmRestoreCompleted(vmRestore *snapshotv1.VirtualMachineRestore) bool {
+	return vmRestore.Status != nil && vmRestore.Status.Complete != nil && *vmRestore.Status.Complete
+}
+
 func VmRestoreProgressing(vmRestore *snapshotv1.VirtualMachineRestore) bool {
-	return vmRestore.Status == nil || vmRestore.Status.Complete == nil || !*vmRestore.Status.Complete
+	return !vmRestoreCompleted(vmRestore) && !vmRestoreFailed(vmRestore)
 }
 
 func vmRestoreDeleting(vmRestore *snapshotv1.VirtualMachineRestore) bool {
@@ -163,12 +184,35 @@ func (ctrl *VMRestoreController) updateVMRestore(vmRestoreIn *snapshotv1.Virtual
 		}
 	}
 
+	ready, err := target.Ready()
+	if err != nil {
+		logger.Reason(err).Error("Error checking target ready")
+		return 0, ctrl.doUpdateError(vmRestoreIn, err)
+	}
+	if !ready {
+		return 0, ctrl.handleVMRestoreTargetNotReady(vmRestoreOut, target)
+	}
+
+	vmSnapshot, err := ctrl.getVMSnapshot(vmRestoreOut)
+	if err != nil {
+		return 0, ctrl.doUpdateError(vmRestoreIn, err)
+	}
+
+	// Check if target exists before the restore
+	// and that it is not the same as the source
+	// We do not allow restoring to an existing
+	// target which is not the same as the source
+	if target.Exists() && !target.TargetRestored() && sourceAndTargetAreDifferent(target, vmSnapshot) {
+		logger.Error(errorRestoreToExistingTarget)
+		return 0, ctrl.doUpdateError(vmRestoreIn, fmt.Errorf(errorRestoreToExistingTarget))
+	}
+
 	err = target.UpdateRestoreInProgress()
 	if err != nil {
 		return 0, err
 	}
 
-	updated, err := ctrl.reconcileVolumeRestores(vmRestoreOut, target)
+	updated, err := ctrl.reconcileVolumeRestores(vmRestoreOut, target, vmSnapshot)
 	if err != nil {
 		logger.Reason(err).Error("Error reconciling VolumeRestores")
 		return 0, ctrl.doUpdateError(vmRestoreIn, err)
@@ -179,19 +223,6 @@ func (ctrl *VMRestoreController) updateVMRestore(vmRestoreIn *snapshotv1.Virtual
 		return 0, ctrl.doUpdateStatus(vmRestoreIn, vmRestoreOut)
 	}
 
-	ready, err := target.Ready()
-	if err != nil {
-		logger.Reason(err).Error("Error checking target ready")
-		return 0, ctrl.doUpdateError(vmRestoreIn, err)
-	}
-	if !ready {
-		reason := "Waiting for target to be ready"
-		updateRestoreCondition(vmRestoreOut, newProgressingCondition(corev1.ConditionFalse, reason))
-		updateRestoreCondition(vmRestoreOut, newReadyCondition(corev1.ConditionFalse, reason))
-		// try again in 5 secs
-		return 5 * time.Second, ctrl.doUpdateStatus(vmRestoreIn, vmRestoreOut)
-	}
-
 	updated, err = target.Reconcile()
 	if err != nil {
 		logger.Reason(err).Error("Error reconciling target")
@@ -293,8 +324,50 @@ func (ctrl *VMRestoreController) handleVMRestoreDeletion(vmRestore *snapshotv1.V
 	return err
 }
 
-func (ctrl *VMRestoreController) reconcileVolumeRestores(vmRestore *snapshotv1.VirtualMachineRestore, target restoreTarget) (bool, error) {
-	content, err := ctrl.getSnapshotContent(vmRestore)
+func (ctrl *VMRestoreController) handleVMRestoreTargetNotReady(vmRestore *snapshotv1.VirtualMachineRestore, target restoreTarget) error {
+	vmRestoreCpy := vmRestore.DeepCopy()
+
+	// default policy is having a grace period for the user the make
+	// the target ready
+	policy := snapshotv1.VirtualMachineRestoreWaitGracePeriodAndFail
+	if vmRestore.Spec.TargetReadinessPolicy != nil {
+		policy = *vmRestore.Spec.TargetReadinessPolicy
+	}
+
+	eventMsg := "Restore target VMI still exists"
+	ctrl.Recorder.Event(vmRestoreCpy, corev1.EventTypeNormal, restoreVMNotReadyEvent, eventMsg)
+	reason := "Waiting for target to be ready"
+	updateRestoreCondition(vmRestoreCpy, newProgressingCondition(corev1.ConditionFalse, reason))
+	updateRestoreCondition(vmRestoreCpy, newReadyCondition(corev1.ConditionFalse, reason))
+
+	switch policy {
+	case snapshotv1.VirtualMachineRestoreWaitEventually:
+	case snapshotv1.VirtualMachineRestoreStopTarget:
+		err := target.Stop()
+		if err != nil {
+			return ctrl.doUpdateError(vmRestore, err)
+		}
+	case snapshotv1.VirtualMachineRestoreWaitGracePeriodAndFail:
+		if vmRestoreTargetReadyGracePeriodExceeded(vmRestore) {
+			updateRestoreCondition(vmRestoreCpy, newProgressingCondition(corev1.ConditionFalse, restoredFailed))
+			updateRestoreCondition(vmRestoreCpy, newFailureCondition(corev1.ConditionTrue, restoreGracePeriodExceededError))
+			updateRestoreCondition(vmRestoreCpy, newReadyCondition(corev1.ConditionFalse, restoredFailed))
+		}
+	case snapshotv1.VirtualMachineRestoreFailImmediate:
+		updateRestoreCondition(vmRestoreCpy, newProgressingCondition(corev1.ConditionFalse, restoredFailed))
+		updateRestoreCondition(vmRestoreCpy, newFailureCondition(corev1.ConditionTrue, restoreTargetNotReady))
+		updateRestoreCondition(vmRestoreCpy, newReadyCondition(corev1.ConditionFalse, restoredFailed))
+	}
+	return ctrl.doUpdateStatus(vmRestore, vmRestoreCpy)
+}
+
+func vmRestoreTargetReadyGracePeriodExceeded(vmRestore *snapshotv1.VirtualMachineRestore) bool {
+	deadline := vmRestore.CreationTimestamp.Add(snapshotv1.DefaultGracePeriod)
+	return time.Until(deadline) < 0
+}
+
+func (ctrl *VMRestoreController) reconcileVolumeRestores(vmRestore *snapshotv1.VirtualMachineRestore, target restoreTarget, vmSnapshot *snapshotv1.VirtualMachineSnapshot) (bool, error) {
+	content, err := ctrl.getSnapshotContent(vmSnapshot)
 	if err != nil {
 		return false, err
 	}
@@ -440,6 +513,15 @@ func (t *vmRestoreTarget) UpdateRestoreInProgress() error {
 	return nil
 }
 
+func (t *vmRestoreTarget) Stop() error {
+	if !t.Exists() {
+		return nil
+	}
+
+	log.Log.Infof("Stopping VM before restore [%s/%s]", t.vm.Namespace, t.vm.Name)
+	return t.controller.Client.VirtualMachine(t.vm.Namespace).Stop(context.Background(), t.vm.Name, &kubevirtv1.StopOptions{})
+}
+
 func (t *vmRestoreTarget) Ready() (bool, error) {
 	if !t.Exists() {
 		return true, nil
@@ -453,28 +535,19 @@ func (t *vmRestoreTarget) Ready() (bool, error) {
 	}
 
 	_, exists, err := t.controller.VMIInformer.GetStore().GetByKey(vmiKey)
-	if err != nil {
-		return false, err
-	}
 
-	return !exists, nil
+	return !exists, err
 }
 
 func (t *vmRestoreTarget) Reconcile() (bool, error) {
 	if t.Exists() && hasLastRestoreAnnotation(t.vmRestore, t.vm) {
 		return false, nil
 	}
-
-	content, err := t.controller.getSnapshotContent(t.vmRestore)
+	snapshotVM, err := t.getSnapshotVM()
 	if err != nil {
 		return false, err
 	}
 
-	snapshotVM := content.Spec.Source.VirtualMachine
-	if snapshotVM == nil {
-		return false, fmt.Errorf("unexpected snapshot source")
-	}
-
 	if updated, err := t.updateVMRestoreRestores(snapshotVM); updated || err != nil {
 		return updated, err
 	}
@@ -494,52 +567,6 @@ func (t *vmRestoreTarget) Reconcile() (bool, error) {
 	return t.reconcileSpec(restoredVM)
 }
 
-func (t *vmRestoreTarget) updateVMRestoreRestores(snapshotVM *snapshotv1.VirtualMachine) (bool, error) {
-	var restores = make([]snapshotv1.VolumeRestore, len(t.vmRestore.Status.Restores))
-	for i, t := range t.vmRestore.Status.Restores {
-		t.DeepCopyInto(&restores[i])
-	}
-	for k := range restores {
-		restore := &restores[k]
-		// Just need to access the regular VM volumes here as the backend volume
-		// is handled separately.
-		volumes, err := storageutils.GetVolumes(snapshotVM, t.controller.Client)
-		if err != nil {
-			return false, err
-		}
-		for _, volume := range volumes {
-			if volume.Name != restore.VolumeName {
-				continue
-			}
-			if volume.DataVolume != nil {
-				templateIndex := findDVTemplateIndex(volume.DataVolume.Name, snapshotVM)
-				if templateIndex >= 0 {
-					dvName := restoreDVName(t.vmRestore, restore.VolumeName)
-					pvc, err := t.controller.getPVC(t.vmRestore.Namespace, restore.PersistentVolumeClaimName)
-					if err != nil {
-						return false, err
-					}
-
-					if pvc == nil {
-						return false, fmt.Errorf("pvc %s/%s does not exist and should", t.vmRestore.Namespace, restore.PersistentVolumeClaimName)
-					}
-
-					if err = t.updatePVCPopulatedForAnnotation(pvc, dvName); err != nil {
-						return false, err
-					}
-					restore.DataVolumeName = &dvName
-					break
-				}
-			}
-		}
-	}
-	if !equality.Semantic.DeepEqual(t.vmRestore.Status.Restores, restores) {
-		t.vmRestore.Status.Restores = restores
-		return true, nil
-	}
-	return false, nil
-}
-
 func (t *vmRestoreTarget) reconcileBackendVolume(snapshotVM *snapshotv1.VirtualMachine) (bool, error) {
 	if !backendstorage.IsBackendStorageNeededForVMI(&snapshotVM.Spec.Template.Spec) {
 		return true, nil
@@ -661,13 +688,77 @@ func getCleanupLabelValue(vmRestore *snapshotv1.VirtualMachineRestore) string {
 	return naming.GetName(backendstorage.PVCPrefix, vmRestore.Spec.Target.Name, validation.DNS1035LabelMaxLength)
 }
 
+func (t *vmRestoreTarget) getSnapshotVM() (*snapshotv1.VirtualMachine, error) {
+	vmSnapshot, err := t.controller.getVMSnapshot(t.vmRestore)
+	if err != nil {
+		return nil, err
+	}
+
+	content, err := t.controller.getSnapshotContent(vmSnapshot)
+	if err != nil {
+		return nil, err
+	}
+
+	snapshotVM := content.Spec.Source.VirtualMachine
+	if snapshotVM == nil {
+		return nil, fmt.Errorf("unexpected snapshot source")
+	}
+
+	return snapshotVM, nil
+}
+
+func (t *vmRestoreTarget) updateVMRestoreRestores(snapshotVM *snapshotv1.VirtualMachine) (bool, error) {
+	var restores = make([]snapshotv1.VolumeRestore, len(t.vmRestore.Status.Restores))
+	for i, t := range t.vmRestore.Status.Restores {
+		t.DeepCopyInto(&restores[i])
+	}
+	for k := range restores {
+		restore := &restores[k]
+		// Just need to access the regular VM volumes here as the backend volume
+		// is handled separately.
+		volumes, err := storageutils.GetVolumes(snapshotVM, t.controller.Client)
+		if err != nil {
+			return false, err
+		}
+		for _, volume := range volumes {
+			if volume.Name != restore.VolumeName {
+				continue
+			}
+			if volume.DataVolume != nil {
+				templateIndex := findDVTemplateIndex(volume.DataVolume.Name, snapshotVM)
+				if templateIndex >= 0 {
+					dvName := restoreDVName(t.vmRestore, restore.VolumeName)
+					pvc, err := t.controller.getPVC(t.vmRestore.Namespace, restore.PersistentVolumeClaimName)
+					if err != nil {
+						return false, err
+					}
+
+					if pvc == nil {
+						return false, fmt.Errorf("pvc %s/%s does not exist and should", t.vmRestore.Namespace, restore.PersistentVolumeClaimName)
+					}
+
+					if err = t.updatePVCPopulatedForAnnotation(pvc, dvName); err != nil {
+						return false, err
+					}
+					restore.DataVolumeName = &dvName
+					break
+				}
+			}
+		}
+	}
+	if !equality.Semantic.DeepEqual(t.vmRestore.Status.Restores, restores) {
+		t.vmRestore.Status.Restores = restores
+		return true, nil
+	}
+	return false, nil
+}
+
 func (t *vmRestoreTarget) UpdateTarget(obj metav1.Object) {
 	t.vm = obj.(*kubevirtv1.VirtualMachine)
 }
 
 func (t *vmRestoreTarget) generateRestoredVMSpec(snapshotVM *snapshotv1.VirtualMachine) (*kubevirtv1.VirtualMachine, error) {
 	log.Log.Object(t.vmRestore).V(3).Info("generating restored VM spec")
-
 	var newTemplates = make([]kubevirtv1.DataVolumeTemplateSpec, len(snapshotVM.Spec.DataVolumeTemplates))
 	var newVolumes []kubevirtv1.Volume
 
@@ -739,28 +830,20 @@ func (t *vmRestoreTarget) generateRestoredVMSpec(snapshotVM *snapshotv1.VirtualM
 			Spec:   *snapshotVM.Spec.DeepCopy(),
 			Status: kubevirtv1.VirtualMachineStatus{},
 		}
-		if newVM.Spec.Running != nil {
-			newVM.Spec.Running = pointer.P(false)
-		} else {
-			newVM.Spec.RunStrategy = pointer.P(kubevirtv1.RunStrategyHalted)
-		}
+
 	} else {
 		newVM = t.vm.DeepCopy()
 		newVM.Spec = *snapshotVM.Spec.DeepCopy()
-		if t.vm.Spec.Running != nil {
-			newVM.Spec.Running = pointer.P(false)
-			newVM.Spec.RunStrategy = nil
-		} else {
-			runStrategy, err := t.vm.RunStrategy()
-			if err != nil {
-				return nil, err
-			}
-			// make sure an existing VM keeps the same run strategy as before the restore
-			newVM.Spec.RunStrategy = pointer.P(runStrategy)
-			newVM.Spec.Running = nil
-		}
 	}
 
+	// update Running state in case snapshot was on online VM
+	if newVM.Spec.RunStrategy != nil {
+		runStrategyHalted := kubevirtv1.RunStrategyHalted
+		newVM.Spec.RunStrategy = &runStrategyHalted
+	} else if newVM.Spec.Running != nil {
+		running := false
+		newVM.Spec.Running = &running
+	}
 	newVM.Spec.DataVolumeTemplates = newTemplates
 	newVM.Spec.Template.Spec.Volumes = newVolumes
 	setLastRestoreAnnotation(t.vmRestore, newVM)
@@ -1088,6 +1171,13 @@ func (ctrl *VMRestoreController) deleteObsoleteBackendPVC(vmRestore *snapshotv1.
 	return nil
 }
 
+func (t *vmRestoreTarget) TargetRestored() bool {
+	return t.Exists() && hasLastRestoreAnnotation(t.vmRestore, t.vm)
+}
+func (t *vmRestoreTarget) UID() types.UID {
+	return t.vm.UID
+}
+
 func (t *vmRestoreTarget) Exists() bool {
 	return t.vm != nil
 }
@@ -1096,7 +1186,12 @@ func (t *vmRestoreTarget) VirtualMachine() *kubevirtv1.VirtualMachine {
 	return t.vm
 }
 
-func (ctrl *VMRestoreController) getSnapshotContent(vmRestore *snapshotv1.VirtualMachineRestore) (*snapshotv1.VirtualMachineSnapshotContent, error) {
+func sourceAndTargetAreDifferent(target restoreTarget, vmSnapshot *snapshotv1.VirtualMachineSnapshot) bool {
+	targetUID := target.UID()
+	return vmSnapshot.Status != nil && vmSnapshot.Status.SourceUID != nil && targetUID != *vmSnapshot.Status.SourceUID
+}
+
+func (ctrl *VMRestoreController) getVMSnapshot(vmRestore *snapshotv1.VirtualMachineRestore) (*snapshotv1.VirtualMachineSnapshot, error) {
 	objKey := cacheKeyFunc(vmRestore.Namespace, vmRestore.Spec.VirtualMachineSnapshotName)
 	obj, exists, err := ctrl.VMSnapshotInformer.GetStore().GetByKey(objKey)
 	if err != nil {
@@ -1107,17 +1202,22 @@ func (ctrl *VMRestoreController) getSnapshotContent(vmRestore *snapshotv1.Virtua
 		return nil, fmt.Errorf("VMSnapshot %s does not exist", objKey)
 	}
 
-	vms := obj.(*snapshotv1.VirtualMachineSnapshot).DeepCopy()
-	if !VmSnapshotReady(vms) {
+	vmSnapshot := obj.(*snapshotv1.VirtualMachineSnapshot).DeepCopy()
+	if vmSnapshotFailed(vmSnapshot) {
+		return nil, fmt.Errorf("VMSnapshot %s failed and is invalid to use", objKey)
+	} else if !VmSnapshotReady(vmSnapshot) {
 		return nil, fmt.Errorf("VMSnapshot %s not ready", objKey)
 	}
 
-	if vms.Status.VirtualMachineSnapshotContentName == nil {
+	if vmSnapshot.Status.VirtualMachineSnapshotContentName == nil {
 		return nil, fmt.Errorf("no snapshot content name in %s", objKey)
 	}
+	return vmSnapshot, nil
+}
 
-	objKey = cacheKeyFunc(vmRestore.Namespace, *vms.Status.VirtualMachineSnapshotContentName)
-	obj, exists, err = ctrl.VMSnapshotContentInformer.GetStore().GetByKey(objKey)
+func (ctrl *VMRestoreController) getSnapshotContent(vmSnapshot *snapshotv1.VirtualMachineSnapshot) (*snapshotv1.VirtualMachineSnapshotContent, error) {
+	objKey := cacheKeyFunc(vmSnapshot.Namespace, *vmSnapshot.Status.VirtualMachineSnapshotContentName)
+	obj, exists, err := ctrl.VMSnapshotContentInformer.GetStore().GetByKey(objKey)
 	if err != nil {
 		return nil, err
 	}
@@ -1126,12 +1226,12 @@ func (ctrl *VMRestoreController) getSnapshotContent(vmRestore *snapshotv1.Virtua
 		return nil, fmt.Errorf("VMSnapshotContent %s does not exist", objKey)
 	}
 
-	vmss := obj.(*snapshotv1.VirtualMachineSnapshotContent).DeepCopy()
-	if !vmSnapshotContentReady(vmss) {
+	vmSnapshotContent := obj.(*snapshotv1.VirtualMachineSnapshotContent).DeepCopy()
+	if !vmSnapshotContentReady(vmSnapshotContent) {
 		return nil, fmt.Errorf("VMSnapshotContent %s not ready", objKey)
 	}
 
-	return vmss, nil
+	return vmSnapshotContent, nil
 }
 
 func (ctrl *VMRestoreController) getVM(namespace, name string) (vm *kubevirtv1.VirtualMachine, err error) {
@@ -1367,7 +1467,7 @@ func CreateRestorePVCDefFromVMRestore(vmRestoreName, restorePVCName string, volu
 }
 
 func updateRestoreCondition(r *snapshotv1.VirtualMachineRestore, c snapshotv1.Condition) {
-	r.Status.Conditions = updateCondition(r.Status.Conditions, c, true)
+	r.Status.Conditions = updateCondition(r.Status.Conditions, c)
 }
 
 // Returns a set of volumes not for restore
