@@ -45,7 +45,7 @@ import (
 	"kubevirt.io/kubevirt/pkg/pointer"
 	"kubevirt.io/kubevirt/pkg/testutils"
 	"kubevirt.io/kubevirt/pkg/virt-api/webhooks"
-	virtconfig "kubevirt.io/kubevirt/pkg/virt-config"
+	"kubevirt.io/kubevirt/pkg/virt-config/featuregate"
 	nodelabellerutil "kubevirt.io/kubevirt/pkg/virt-handler/node-labeller/util"
 	"kubevirt.io/kubevirt/pkg/virt-operator/resource/generate/components"
 )
@@ -589,8 +589,8 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 			Expect(exist).To(BeFalse())
 		},
 		Entry("when the AlignCPUs featureGate is disabled", "", map[string]string{v1.EmulatorThreadCompleteToEvenParity: ""}, true),
-		Entry("when the EmulatorThreadCompleteToEvenParity annotation is not set on the kubevirt CR", virtconfig.AlignCPUsGate, nil, true),
-		Entry("when isolateEmulatorThread is disabled on the VMI spec", virtconfig.AlignCPUsGate, map[string]string{v1.EmulatorThreadCompleteToEvenParity: ""}, false),
+		Entry("when the EmulatorThreadCompleteToEvenParity annotation is not set on the kubevirt CR", featuregate.AlignCPUsGate, nil, true),
+		Entry("when isolateEmulatorThread is disabled on the VMI spec", featuregate.AlignCPUsGate, map[string]string{v1.EmulatorThreadCompleteToEvenParity: ""}, false),
 	)
 
 	It("should copy the EmulatorThreadCompleteToEvenParity annotation to the VMI", func() {
@@ -601,7 +601,7 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 			Spec: v1.KubeVirtSpec{
 				Configuration: v1.KubeVirtConfiguration{
 					DeveloperConfiguration: &v1.DeveloperConfiguration{
-						FeatureGates: []string{virtconfig.AlignCPUsGate},
+						FeatureGates: []string{featuregate.AlignCPUsGate},
 					},
 				},
 			},
@@ -1136,7 +1136,7 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 				Spec: v1.KubeVirtSpec{
 					Configuration: v1.KubeVirtConfiguration{
 						DeveloperConfiguration: &v1.DeveloperConfiguration{
-							FeatureGates: []string{virtconfig.Root},
+							FeatureGates: []string{featuregate.Root},
 						},
 					},
 				},
@@ -1153,113 +1153,6 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 		Expect(status.RuntimeUser).NotTo(BeZero())
 	})
 
-	It("should add realtime node label selector with realtime workload", func() {
-		vmi.Spec.Domain.CPU = &v1.CPU{Realtime: &v1.Realtime{}}
-		_, vmiSpec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
-		Expect(vmiSpec.NodeSelector).NotTo(BeNil())
-		Expect(vmiSpec.NodeSelector).To(BeEquivalentTo(map[string]string{v1.RealtimeLabel: ""}))
-	})
-	It("should not add realtime node label selector when no realtime workload", func() {
-		vmi.Spec.Domain.CPU = &v1.CPU{Realtime: nil}
-		vmi.Spec.NodeSelector = map[string]string{v1.NodeSchedulable: "true"}
-		_, vmiSpec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
-		Expect(vmiSpec.NodeSelector).To(BeEquivalentTo(map[string]string{v1.NodeSchedulable: "true"}))
-	})
-	It("should not overwrite existing node label selectors with realtime workload", func() {
-		vmi.Spec.Domain.CPU = &v1.CPU{Realtime: &v1.Realtime{}}
-		vmi.Spec.NodeSelector = map[string]string{v1.NodeSchedulable: "true"}
-		_, vmiSpec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
-		Expect(vmiSpec.NodeSelector).To(BeEquivalentTo(map[string]string{v1.NodeSchedulable: "true", v1.RealtimeLabel: ""}))
-	})
-
-	DescribeTable("When scheduling SEV workloads",
-		func(nodeSelectorBefore map[string]string,
-			nodeSelectorAfter map[string]string,
-			launchSec *v1.LaunchSecurity) {
-			vmi.Spec.NodeSelector = nodeSelectorBefore
-			vmi.Spec.Domain.LaunchSecurity = launchSec
-			_, vmiSpec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
-			Expect(vmiSpec.NodeSelector).NotTo(BeNil())
-			Expect(vmiSpec.NodeSelector).To(BeEquivalentTo(nodeSelectorAfter))
-		},
-		Entry("It should add SEV node label selector with SEV workload",
-			map[string]string{},
-			map[string]string{v1.SEVLabel: ""},
-			&v1.LaunchSecurity{SEV: &v1.SEV{}}),
-		Entry("It should not add SEV node label selector when no SEV workload",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{v1.NodeSchedulable: "true"},
-			&v1.LaunchSecurity{}),
-		Entry("It should not overwrite existing node label selectors with SEV workload",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{v1.NodeSchedulable: "true", v1.SEVLabel: ""},
-			&v1.LaunchSecurity{SEV: &v1.SEV{}}),
-		Entry("It should add SEV and SEV-ES node label selector with SEV-ES workload",
-			map[string]string{},
-			map[string]string{
-				v1.SEVLabel:   "",
-				v1.SEVESLabel: "",
-			},
-			&v1.LaunchSecurity{
-				SEV: &v1.SEV{
-					Policy: &v1.SEVPolicy{
-						EncryptedState: pointer.P(true),
-					},
-				},
-			}),
-		Entry("It should not add SEV-ES node label selector when no SEV policy is set",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{
-				v1.NodeSchedulable: "true",
-				v1.SEVLabel:        "",
-			},
-			&v1.LaunchSecurity{
-				SEV: &v1.SEV{
-					Policy: &v1.SEVPolicy{},
-				},
-			}),
-		Entry("It should not add SEV-ES node label selector when no SEV-ES policy bit is set",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{
-				v1.NodeSchedulable: "true",
-				v1.SEVLabel:        "",
-			},
-			&v1.LaunchSecurity{
-				SEV: &v1.SEV{
-					Policy: &v1.SEVPolicy{
-						EncryptedState: nil,
-					},
-				},
-			}),
-		Entry("It should not add SEV-ES node label selector when SEV-ES policy bit is set to false",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{
-				v1.NodeSchedulable: "true",
-				v1.SEVLabel:        "",
-			},
-			&v1.LaunchSecurity{
-				SEV: &v1.SEV{
-					Policy: &v1.SEVPolicy{
-						EncryptedState: pointer.P(false),
-					},
-				},
-			}),
-		Entry("It should not overwrite existing node label selectors with SEV-ES workload",
-			map[string]string{v1.NodeSchedulable: "true"},
-			map[string]string{
-				v1.NodeSchedulable: "true",
-				v1.SEVLabel:        "",
-				v1.SEVESLabel:      "",
-			},
-			&v1.LaunchSecurity{
-				SEV: &v1.SEV{
-					Policy: &v1.SEVPolicy{
-						EncryptedState: pointer.P(true),
-					},
-				},
-			}),
-	)
-
 	DescribeTable("evictionStrategy should match the", func(f func(*v1.VirtualMachineInstanceSpec) v1.EvictionStrategy) {
 		expected := f(&vmi.Spec)
 		_, vmiSpec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
@@ -1362,14 +1255,11 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 		})
 	})
 
-	Context("when LiveUpdate and VMLiveUpdateFeatures is enabled", func() {
+	Context("when vmRolloutStrategy LiveUpdate is enabled", func() {
 		BeforeEach(func() {
 			kvCR := testutils.GetFakeKubeVirtClusterConfig(kvStore)
 			rolloutStrategy := v1.VMRolloutStrategyLiveUpdate
 			kvCR.Spec.Configuration.VMRolloutStrategy = &rolloutStrategy
-			kvCR.Spec.Configuration.DeveloperConfiguration = &v1.DeveloperConfiguration{
-				FeatureGates: []string{virtconfig.VMLiveUpdateFeaturesGate},
-			}
 			testutils.UpdateFakeKubeVirtClusterConfig(kvStore, kvCR)
 		})
 		Context("configure CPU hotplug", func() {
@@ -1423,17 +1313,6 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 				_, spec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
 				Expect(spec.Domain.CPU.MaxSockets).To(Equal(uint32(8)))
 			})
-
-			It("to calculate max sockets to be 4x times the configured sockets with upper bound 512 when no max sockets defined", func() {
-				vmi.Spec.Domain.CPU = &v1.CPU{
-					Sockets: 32,
-					Cores:   2,
-					Threads: 3,
-				}
-				_, spec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
-				Expect(spec.Domain.CPU.MaxSockets).To(Equal(uint32(85)))
-			})
-
 			It("to calculate max sockets to be 4x times the default sockets when default CPU topology used", func() {
 				_, spec, _ := getMetaSpecStatusFromAdmit(rt.GOARCH)
 				Expect(spec.Domain.CPU.MaxSockets).To(Equal(uint32(4)))
@@ -1600,6 +1479,9 @@ var _ = Describe("VirtualMachineInstance Mutator", func() {
 				Entry("architecture is not amd64 or arm64", func(vmiSpec *v1.VirtualMachineInstanceSpec) {
 					vmiSpec.Architecture = "risc-v"
 				}),
+				Entry("guest memory is less than 1Gi", func(vmiSpec *v1.VirtualMachineInstanceSpec) {
+					vmiSpec.Domain.Memory.Guest = pointer.P(resource.MustParse("512Mi"))
+				}),
 			)
 		})
 	})
