@@ -5,7 +5,6 @@ import (
 	"path/filepath"
 
 	backendstorage "kubevirt.io/kubevirt/pkg/storage/backend-storage"
-	"kubevirt.io/kubevirt/pkg/tpm"
 
 	k8sv1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
@@ -26,14 +25,15 @@ import (
 type VolumeRendererOption func(renderer *VolumeRenderer) error
 
 type VolumeRenderer struct {
-	containerDiskDir string
-	ephemeralDiskDir string
-	virtShareDir     string
-	namespace        string
-	vmiVolumes       []v1.Volume
-	podVolumes       []k8sv1.Volume
-	podVolumeMounts  []k8sv1.VolumeMount
-	volumeDevices    []k8sv1.VolumeDevice
+	containerDiskDir      string
+	ephemeralDiskDir      string
+	virtShareDir          string
+	namespace             string
+	vmiVolumes            []v1.Volume
+	podVolumes            []k8sv1.Volume
+	podVolumeMounts       []k8sv1.VolumeMount
+	sharedFilesystemPaths []string
+	volumeDevices         []k8sv1.VolumeDevice
 }
 
 func NewVolumeRenderer(namespace string, ephemeralDisk string, containerDiskDir string, virtShareDir string, volumeOptions ...VolumeRendererOption) (*VolumeRenderer, error) {
@@ -80,6 +80,10 @@ func (vr *VolumeRenderer) VolumeDevices() []k8sv1.VolumeDevice {
 	return vr.volumeDevices
 }
 
+func (vr *VolumeRenderer) SharedFilesystemPaths() []string {
+	return vr.sharedFilesystemPaths
+}
+
 func mountPath(name string, path string) k8sv1.VolumeMount {
 	return k8sv1.VolumeMount{
 		Name:      name,
@@ -380,6 +384,13 @@ func withBackendStorage(vmi *v1.VirtualMachineInstance, backendStoragePVCName st
 			},
 		})
 
+		renderer.podVolumeMounts = append(renderer.podVolumeMounts, k8sv1.VolumeMount{
+			Name:      volumeName,
+			ReadOnly:  false,
+			MountPath: "/run/kubevirt-private/backend-storage-meta",
+			SubPath:   "meta",
+		})
+
 		if util.IsNonRootVMI(vmi) {
 			// For non-root VMIs, the TPM state lives under /var/run/kubevirt-private/libvirt/qemu/swtpm
 			// To persist it, we need the persistent PVC to be mounted under that location.
@@ -401,7 +412,7 @@ func withBackendStorage(vmi *v1.VirtualMachineInstance, backendStoragePVCName st
 			})
 		}
 
-		if tpm.HasPersistentDevice(&vmi.Spec) {
+		if backendstorage.HasPersistentTPMDevice(&vmi.Spec) {
 			renderer.podVolumeMounts = append(renderer.podVolumeMounts, k8sv1.VolumeMount{
 				Name:      volumeName,
 				ReadOnly:  false,
@@ -526,7 +537,7 @@ func serviceAccount(volumes ...v1.Volume) string {
 
 func (vr *VolumeRenderer) addPVCToLaunchManifest(pvcStore cache.Store, volume v1.Volume, claimName string) error {
 	logger := log.DefaultLogger()
-	_, exists, isBlock, err := types.IsPVCBlockFromStore(pvcStore, vr.namespace, claimName)
+	pvc, exists, isBlock, err := types.IsPVCBlockFromStore(pvcStore, vr.namespace, claimName)
 	if err != nil {
 		logger.Errorf("error getting PVC: %v", claimName)
 		return err
@@ -541,11 +552,15 @@ func (vr *VolumeRenderer) addPVCToLaunchManifest(pvcStore cache.Store, volume v1
 		}
 		vr.volumeDevices = append(vr.volumeDevices, device)
 	} else {
+		path := hostdisk.GetMountedHostDiskDir(volume.Name)
 		volumeMount := k8sv1.VolumeMount{
 			Name:      volume.Name,
-			MountPath: hostdisk.GetMountedHostDiskDir(volume.Name),
+			MountPath: path,
 		}
 		vr.podVolumeMounts = append(vr.podVolumeMounts, volumeMount)
+		if types.HasSharedAccessMode(pvc.Spec.AccessModes) {
+			vr.sharedFilesystemPaths = append(vr.sharedFilesystemPaths, path)
+		}
 	}
 	return nil
 }
