@@ -64,7 +64,6 @@ import (
 	storagetypes "kubevirt.io/kubevirt/pkg/storage/types"
 	"kubevirt.io/kubevirt/pkg/virt-controller/services"
 	"kubevirt.io/kubevirt/pkg/virt-controller/watch/descheduler"
-	"kubevirt.io/kubevirt/pkg/virt-controller/watch/migration/priorityqueue"
 )
 
 const (
@@ -91,20 +90,12 @@ const defaultFinalizedMigrationGarbageCollectionBuffer = 5
 // cause the migration to fail when it could have reasonably succeeded.
 const defaultCatchAllPendingTimeoutSeconds = int64(60 * 15)
 
-// This controller is driven by a priority queue, so that proper attention is
-// given to active migrations. When a pending migration gets re-enqueued for
-// capacity reasons, we need to ensure it doesn't get re-processed as long as
-// capacity hasn't freed up, or it will delay processing of active migrations.
-// Active migrations default to a priority of 0. -100 leaves plenty of room
-// for potential future "semi-low" priority values.
-const lowPriority = -100
-
 var migrationBackoffError = errors.New(controller.MigrationBackoffReason)
 
 type Controller struct {
 	templateService      services.TemplateService
 	clientset            kubecli.KubevirtClient
-	Queue                priorityqueue.PriorityQueue[string]
+	Queue                workqueue.TypedRateLimitingInterface[string]
 	vmiStore             cache.Store
 	podIndexer           cache.Indexer
 	migrationIndexer     cache.Indexer
@@ -149,9 +140,10 @@ func NewController(templateService services.TemplateService,
 
 	c := &Controller{
 		templateService: templateService,
-		Queue: priorityqueue.New[string]("virt-controller-migration", func(o *priorityqueue.Opts[string]) {
-			o.RateLimiter = workqueue.DefaultTypedControllerRateLimiter[string]()
-		}),
+		Queue: workqueue.NewTypedRateLimitingQueueWithConfig[string](
+			workqueue.DefaultTypedControllerRateLimiter[string](),
+			workqueue.TypedRateLimitingQueueConfig[string]{Name: "virt-controller-migration"},
+		),
 		vmiStore:             vmiInformer.GetStore(),
 		podIndexer:           podInformer.GetIndexer(),
 		migrationIndexer:     migrationInformer.GetIndexer(),
@@ -249,7 +241,7 @@ func (c *Controller) runWorker() {
 }
 
 func (c *Controller) Execute() bool {
-	key, priority, quit := c.Queue.GetWithPriority()
+	key, quit := c.Queue.Get()
 	if quit {
 		return false
 	}
@@ -258,7 +250,7 @@ func (c *Controller) Execute() bool {
 
 	if err != nil {
 		log.Log.Reason(err).Infof("reenqueuing Migration %v", key)
-		c.Queue.AddWithOpts(priorityqueue.AddOpts{Priority: priority, RateLimited: true}, key)
+		c.Queue.AddRateLimited(key)
 	} else {
 		log.Log.V(4).Infof("processed Migration %v", key)
 		c.Queue.Forget(key)
@@ -341,14 +333,12 @@ func (c *Controller) execute(key string) error {
 	}
 
 	if !vmiExists {
-		var err error
-
 		if migration.DeletionTimestamp == nil {
 			logger.V(3).Infof("Deleting migration for deleted vmi %s/%s", migration.Namespace, migration.Spec.VMIName)
-			err = c.clientset.VirtualMachineInstanceMigration(migration.Namespace).Delete(context.Background(), migration.Name, v1.DeleteOptions{})
+			return c.clientset.VirtualMachineInstanceMigration(migration.Namespace).Delete(context.Background(), migration.Name, v1.DeleteOptions{})
 		}
-		// nothing to process for a migration that's being deleted
-		return err
+		// nothing to process for a migration that has no VMI
+		return nil
 	}
 
 	vmi = vmiObj.(*virtv1.VirtualMachineInstance)
@@ -426,11 +416,19 @@ func (c *Controller) failMigration(migration *virtv1.VirtualMachineInstanceMigra
 		return err
 	}
 	migration.Status.Phase = virtv1.MigrationFailed
+
 	return nil
 }
 
-func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, pods []*k8sv1.Pod, syncError error) error {
+func (c *Controller) interruptMigration(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance) error {
+	if vmi == nil || !backendstorage.IsBackendStorageNeededForVMI(&vmi.Spec) {
+		return c.failMigration(migration)
+	}
 
+	return backendstorage.RecoverFromBrokenMigration(c.clientset, migration, c.pvcStore, vmi, c.templateService.GetLauncherImage())
+}
+
+func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigration, vmi *virtv1.VirtualMachineInstance, pods []*k8sv1.Pod, syncError error) error {
 	var pod *k8sv1.Pod = nil
 	var attachmentPod *k8sv1.Pod = nil
 	conditionManager := controller.NewVirtualMachineInstanceMigrationConditionManager()
@@ -450,22 +448,19 @@ func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigrat
 		}
 	}
 
-	// Remove the finalizer and conditions if the migration has already completed
+	// Status checking of active Migration job.
+	//
+	// - Fail if any obvious failure is found
+	// - Interrupt if something unexpectedly disappeared
+	// - Begin progressing migration state based on VMI's MigrationState status.
 	if migration.IsFinal() {
-
 		if vmi.Status.MigrationState != nil && migration.UID == vmi.Status.MigrationState.MigrationUID {
 			// Store the finalized migration state data from the VMI status in the migration object
 			migrationCopy.Status.MigrationState = vmi.Status.MigrationState
 		}
 
-		// remove the migration finalizer
+		// Remove the finalizer and conditions if the migration has already completed
 		controller.RemoveFinalizer(migrationCopy, virtv1.VirtualMachineInstanceMigrationFinalizer)
-
-		// Status checking of active Migration job.
-		//
-		// 1. Fail if VMI isn't in running state.
-		// 2. Fail if target pod exists and has gone down for any reason.
-		// 3. Begin progressing migration state based on VMI's MigrationState status.
 	} else if vmi == nil {
 		err := c.failMigration(migrationCopy)
 		if err != nil {
@@ -474,7 +469,7 @@ func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigrat
 		c.recorder.Eventf(migration, k8sv1.EventTypeWarning, controller.FailedMigrationReason, "Migration failed because vmi does not exist.")
 		log.Log.Object(migration).Error("vmi does not exist")
 	} else if vmi.IsFinal() {
-		err := c.failMigration(migrationCopy)
+		err := c.interruptMigration(migrationCopy, vmi)
 		if err != nil {
 			return err
 		}
@@ -495,14 +490,14 @@ func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigrat
 			return err
 		}
 	} else if podExists && controller.PodIsDown(pod) {
-		err := c.failMigration(migrationCopy)
+		err := c.interruptMigration(migrationCopy, vmi)
 		if err != nil {
 			return err
 		}
 		c.recorder.Eventf(migration, k8sv1.EventTypeWarning, controller.FailedMigrationReason, "Migration failed because target pod shutdown during migration")
 		log.Log.Object(migration).Errorf("target pod %s/%s shutdown during migration", pod.Namespace, pod.Name)
 	} else if migration.TargetIsCreated() && !podExists {
-		err := c.failMigration(migrationCopy)
+		err := c.interruptMigration(migrationCopy, vmi)
 		if err != nil {
 			return err
 		}
@@ -566,11 +561,13 @@ func (c *Controller) updateStatus(migration *virtv1.VirtualMachineInstanceMigrat
 	controller.SetSourcePod(migrationCopy, vmi, c.podIndexer)
 
 	if !equality.Semantic.DeepEqual(migration.Status, migrationCopy.Status) {
-		_, err := c.clientset.VirtualMachineInstanceMigration(migrationCopy.Namespace).UpdateStatus(context.Background(), migrationCopy, v1.UpdateOptions{})
+		var err error
+		migration, err = c.clientset.VirtualMachineInstanceMigration(migrationCopy.Namespace).UpdateStatus(context.Background(), migrationCopy, v1.UpdateOptions{})
 		if err != nil {
 			return err
 		}
-	} else if !equality.Semantic.DeepEqual(migration.Finalizers, migrationCopy.Finalizers) {
+	}
+	if !equality.Semantic.DeepEqual(migration.Finalizers, migrationCopy.Finalizers) {
 		_, err := c.clientset.VirtualMachineInstanceMigration(migrationCopy.Namespace).Update(context.Background(), migrationCopy, metav1.UpdateOptions{})
 		if err != nil {
 			return err
@@ -1095,8 +1092,8 @@ func (c *Controller) handleTargetPodCreation(key string, migration *virtv1.Virtu
 	// XXX: Make this configurable, think about limit per node, bandwidth per migration, and so on.
 	if len(runningMigrations) >= int(*c.clusterConfig.GetMigrationConfiguration().ParallelMigrationsPerCluster) {
 		log.Log.Object(migration).Infof("Waiting to schedule target pod for vmi [%s/%s] migration because total running parallel migration count [%d] is currently at the global cluster limit.", vmi.Namespace, vmi.Name, len(runningMigrations))
-		// The controller is busy with active migrations, mark ourselves as low priority to give more cycles to those
-		c.Queue.AddWithOpts(priorityqueue.AddOpts{Priority: lowPriority, After: 5 * time.Second}, key)
+		// Let's wait until some migrations are done
+		c.Queue.AddAfter(key, time.Second*5)
 		return nil
 	}
 
@@ -1110,8 +1107,7 @@ func (c *Controller) handleTargetPodCreation(key string, migration *virtv1.Virtu
 		// Let's ensure that we only have two outbound migrations per node
 		// XXX: Make this configurable, thinkg about inbound migration limit, bandwidh per migration, and so on.
 		log.Log.Object(migration).Infof("Waiting to schedule target pod for vmi [%s/%s] migration because total running parallel outbound migrations on target node [%d] has hit outbound migrations per node limit.", vmi.Namespace, vmi.Name, outboundMigrations)
-		// The controller is busy with active migrations, mark ourselves as low priority to give more cycles to those
-		c.Queue.AddWithOpts(priorityqueue.AddOpts{Priority: lowPriority, After: 5 * time.Second}, key)
+		c.Queue.AddAfter(key, time.Second*5)
 		return nil
 	}
 
@@ -1595,7 +1591,7 @@ func (c *Controller) resolveControllerRef(namespace string, controllerRef *v1.Ow
 	if controllerRef.Kind != virtv1.VirtualMachineInstanceMigrationGroupVersionKind.Kind {
 		return nil
 	}
-	migration, exists, err := c.migrationIndexer.GetByKey(namespace + "/" + controllerRef.Name)
+	migration, exists, err := c.migrationIndexer.GetByKey(controller.NamespacedKey(namespace, controllerRef.Name))
 	if err != nil {
 		return nil
 	}
@@ -1794,7 +1790,7 @@ func (c *Controller) addPVC(obj interface{}) {
 	if !exists {
 		return
 	}
-	migrationKey := pvc.Namespace + "/" + migrationName
+	migrationKey := controller.NamespacedKey(pvc.Namespace, migrationName)
 	c.pvcExpectations.CreationObserved(migrationKey)
 	c.Queue.Add(migrationKey)
 }
@@ -1974,7 +1970,8 @@ func (c *Controller) deleteVMI(obj interface{}) {
 func (c *Controller) outboundMigrationsOnNode(node string, runningMigrations []*virtv1.VirtualMachineInstanceMigration) (int, error) {
 	sum := 0
 	for _, migration := range runningMigrations {
-		if vmi, exists, _ := c.vmiStore.GetByKey(migration.Namespace + "/" + migration.Spec.VMIName); exists {
+		key := controller.NamespacedKey(migration.Namespace, migration.Spec.VMIName)
+		if vmi, exists, _ := c.vmiStore.GetByKey(key); exists {
 			if vmi.(*virtv1.VirtualMachineInstance).Status.NodeName == node {
 				sum = sum + 1
 			}
@@ -1996,7 +1993,8 @@ func (c *Controller) findRunningMigrations() ([]*virtv1.VirtualMachineInstanceMi
 			runningMigrations = append(runningMigrations, migration)
 			continue
 		}
-		vmi, exists, err := c.vmiStore.GetByKey(migration.Namespace + "/" + migration.Spec.VMIName)
+		key := controller.NamespacedKey(migration.Namespace, migration.Spec.VMIName)
+		vmi, exists, err := c.vmiStore.GetByKey(key)
 		if err != nil {
 			return nil, err
 		}
