@@ -134,7 +134,6 @@ type volumeMounter struct {
 	hotplugDiskManager hotplugdisk.HotplugDiskManagerInterface
 	ownershipManager   diskutils.OwnershipManagerInterface
 	kubeletPodsDir     string
-	host               string
 }
 
 // VolumeMounter is the interface used to mount and unmount volumes to/from a running virtlauncher pod.
@@ -160,14 +159,13 @@ type vmiMountTargetRecord struct {
 }
 
 // NewVolumeMounter creates a new VolumeMounter
-func NewVolumeMounter(mountStateDir string, kubeletPodsDir string, host string) VolumeMounter {
+func NewVolumeMounter(mountStateDir string, kubeletPodsDir string) VolumeMounter {
 	return &volumeMounter{
 		mountRecords:       make(map[types.UID]*vmiMountTargetRecord),
 		checkpointManager:  checkpoint.NewSimpleCheckpointManager(mountStateDir),
 		hotplugDiskManager: hotplugdisk.NewHotplugDiskManager(kubeletPodsDir),
 		ownershipManager:   diskutils.DefaultOwnershipManager,
 		kubeletPodsDir:     kubeletPodsDir,
-		host:               host,
 	}
 }
 
@@ -284,7 +282,7 @@ func (m *volumeMounter) mountHotplugVolume(
 ) error {
 	logger := log.DefaultLogger()
 	logger.V(4).Infof("Hotplug check volume name: %s", volumeName)
-	if sourceUID != types.UID("") {
+	if sourceUID != "" {
 		if m.isBlockVolume(&vmi.Status, volumeName) {
 			logger.V(4).Infof("Mounting block volume: %s", volumeName)
 			if err := m.mountBlockHotplugVolume(vmi, volumeName, sourceUID, record, cgroupManager); err != nil {
@@ -305,7 +303,7 @@ func (m *volumeMounter) mountHotplugVolume(
 }
 
 func (m *volumeMounter) Mount(vmi *v1.VirtualMachineInstance, cgroupManager cgroup.Manager) error {
-	return m.mountFromPod(vmi, types.UID(""), cgroupManager)
+	return m.mountFromPod(vmi, "", cgroupManager)
 }
 
 func (m *volumeMounter) MountFromPod(vmi *v1.VirtualMachineInstance, sourceUID types.UID, cgroupManager cgroup.Manager) error {
@@ -326,7 +324,7 @@ func (m *volumeMounter) mountFromPod(vmi *v1.VirtualMachineInstance, sourceUID t
 		if volumeStatus.MemoryDumpVolume != nil {
 			mountDirectory = true
 		}
-		if sourceUID == types.UID("") {
+		if sourceUID == "" {
 			sourceUID = volumeStatus.HotplugVolume.AttachPodUID
 		}
 		if err := m.mountHotplugVolume(vmi, volumeStatus.Name, sourceUID, record, mountDirectory, cgroupManager); err != nil {
@@ -348,22 +346,6 @@ func (m *volumeMounter) isDirectoryMounted(vmiStatus *v1.VirtualMachineInstanceS
 // isBlockVolume checks if the volumeDevices directory exists in the pod path, we assume there is a single volume associated with
 // each pod, we use this knowledge to determine if we have a block volume or not.
 func (m *volumeMounter) isBlockVolume(vmiStatus *v1.VirtualMachineInstanceStatus, volumeName string) bool {
-	// First evaluate the migrated volumed. In the case of a migrated volume, virt-handler needs to understand if it needs to consider the
-	// volume mode of the source or the destination. Therefore, it evaluates if its is running on the destiation host or otherwise, it is the source.
-	isDstHost := false
-	if vmiStatus.MigrationState != nil {
-		isDstHost = vmiStatus.MigrationState.TargetNode == m.host
-	}
-	for _, migVol := range vmiStatus.MigratedVolumes {
-		if migVol.VolumeName == volumeName {
-			if isDstHost && migVol.DestinationPVCInfo != nil {
-				return storagetypes.IsPVCBlock(migVol.DestinationPVCInfo.VolumeMode)
-			}
-			if !isDstHost && migVol.SourcePVCInfo != nil {
-				return storagetypes.IsPVCBlock(migVol.SourcePVCInfo.VolumeMode)
-			}
-		}
-	}
 	// Check if the volumeDevices directory exists in the attachment pod, if so, its a block device, otherwise its file system.
 	for _, status := range vmiStatus.VolumeStatus {
 		if status.Name == volumeName {
@@ -553,7 +535,7 @@ func (m *volumeMounter) findVirtlauncherUID(vmi *v1.VirtualMachineInstance) (uid
 		return
 	}
 	// Either no pods, or multiple pods, skip.
-	return types.UID("")
+	return ""
 }
 
 func (m *volumeMounter) getSourcePodFilePath(sourceUID types.UID, vmi *v1.VirtualMachineInstance, volume string) (*safepath.Path, error) {
