@@ -50,8 +50,6 @@ import (
 
 	k8sv1 "k8s.io/api/core/v1"
 
-	hotplugdisk "kubevirt.io/kubevirt/pkg/hotplug-disk"
-	storagetypes "kubevirt.io/kubevirt/pkg/storage/types"
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/cli"
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/device/hostdevice"
 	"kubevirt.io/kubevirt/pkg/virt-launcher/virtwrap/device/hostdevice/sriov"
@@ -116,7 +114,7 @@ func generateMigrationFlags(isBlockMigration, migratePaused bool, options *cmdcl
 	if migratePaused {
 		migrateFlags |= libvirt.MIGRATE_PAUSED
 	}
-	if options.ParallelMigrationThreads != nil {
+	if shouldConfigureParallel, _ := shouldConfigureParallelMigration(options); shouldConfigureParallel {
 		migrateFlags |= libvirt.MIGRATE_PARALLEL
 	}
 
@@ -442,6 +440,11 @@ func (m *migrationMonitor) isMigrationPostCopy() bool {
 	return migration.Mode == v1.MigrationPostCopy
 }
 
+func (m *migrationMonitor) isPausedMigration() bool {
+	migration, _ := m.l.metadataCache.Migration.Load()
+	return migration.Mode == v1.MigrationPaused
+}
+
 func (m *migrationMonitor) shouldTriggerTimeout(elapsed int64) bool {
 	if m.acceptableCompletionTime == 0 {
 		return false
@@ -450,8 +453,8 @@ func (m *migrationMonitor) shouldTriggerTimeout(elapsed int64) bool {
 	return elapsed/int64(time.Second) > m.acceptableCompletionTime
 }
 
-func (m *migrationMonitor) shouldTriggerPostCopy(elapsed int64) bool {
-	return m.shouldTriggerTimeout(elapsed) && m.options.AllowPostCopy
+func (m *migrationMonitor) shouldAssistMigrationToComplete(elapsed int64) bool {
+	return m.shouldTriggerTimeout(elapsed) && m.options.AllowWorkloadDisruption
 }
 
 func (m *migrationMonitor) isMigrationProgressing() bool {
@@ -533,17 +536,35 @@ func (m *migrationMonitor) processInflightMigration(dom cli.VirDomain, stats *li
 		// If we were to abort the migration due to a timeout while in post copy,
 		// then it would result in that active state being lost.
 
-	case m.shouldTriggerPostCopy(elapsed):
-		logger.Info("Starting post copy mode for migration")
-		// if a migration has stalled too long, post copy will be
-		// triggered when allowPostCopy is enabled
-		err := dom.MigrateStartPostCopy(uint32(0))
-		if err != nil {
-			logger.Reason(err).Error("failed to start post migration")
-			return nil
-		}
+	case m.shouldAssistMigrationToComplete(elapsed) && !m.isPausedMigration():
+		if m.options.AllowPostCopy {
+			logger.Info("Starting post copy mode for migration")
+			// if a migration has stalled too long, post copy will be
+			// triggered when allowPostCopy is enabled
+			err := dom.MigrateStartPostCopy(0)
+			if err != nil {
+				logger.Reason(err).Error("failed to start post migration")
+				return nil
+			}
+			m.l.updateVMIMigrationMode(v1.MigrationPostCopy)
+		} else {
 
-		m.l.updateVMIMigrationMode(v1.MigrationPostCopy)
+			logger.Info("Pausing the guest to allow migration to complete")
+			// if a migration has stalled too long, the guest will be paused
+			// to complete the migration when allowPostCopy is disabled
+			err := dom.Suspend()
+			if err != nil {
+				logger.Reason(err).Error("Signalling suspension failed.")
+				return nil
+			}
+			logger.Infof("Signaled pause for %s", m.vmi.GetObjectMeta().GetName())
+
+			// update acceptableCompletionTime to prevent premature migration
+			// cancellation
+			m.acceptableCompletionTime *= 2
+			m.l.paused.add(m.vmi.UID)
+			m.l.updateVMIMigrationMode(v1.MigrationPaused)
+		}
 
 	case !m.isMigrationProgressing():
 		// check if the migration is still progressing
@@ -621,6 +642,13 @@ func (m *migrationMonitor) startMonitor() {
 			if strings.Contains(m.migrationFailedWithError.Error(), "canceled by client") {
 				abortStatus = v1.MigrationAbortSucceeded
 			}
+			// Improve the error message when the volume migration fails because the destination size is smaller then the source volume
+			if len(vmi.Status.MigratedVolumes) > 0 && strings.Contains(m.migrationFailedWithError.Error(),
+				"has to be smaller or equal to the actual size of the containing file") {
+				m.l.setMigrationResult(true, fmt.Sprintf("Volume migration cannot be performed because the destination volume is smaller then the source volume: %v",
+					m.migrationFailedWithError), abortStatus)
+				return
+			}
 			m.l.setMigrationResult(true, fmt.Sprintf("Live migration failed %v", m.migrationFailedWithError), abortStatus)
 			return
 		}
@@ -728,12 +756,7 @@ func generateMigrationParams(dom cli.VirDomain, vmi *v1.VirtualMachineInstance,
 		return nil, err
 	}
 
-	parallelMigrationSet := false
-	var parallelMigrationThreads int
-	if options.ParallelMigrationThreads != nil {
-		parallelMigrationSet = true
-		parallelMigrationThreads = int(*options.ParallelMigrationThreads)
-	}
+	parallelMigrationSet, parallelMigrationThreads := shouldConfigureParallelMigration(options)
 
 	key := migrationproxy.ConstructProxyKey(string(vmi.UID), migrationproxy.LibvirtDirectMigrationPort)
 	migrURI := fmt.Sprintf("unix://%s", migrationproxy.SourceUnixFile(virtShareDir, key))
@@ -866,13 +889,6 @@ func configureLocalDiskToMigrate(dom *libvirtxml.Domain, vmi *v1.VirtualMachineI
 	migDisks := classifyVolumesForMigration(vmi)
 	fsSrcBlockDstVols := getFsSrcBlockDstVols(vmi)
 	blockSrcFsDstVols := getBlockSrcFsDstVols(vmi)
-	hotplugVols := make(map[string]bool)
-
-	for _, v := range vmi.Spec.Volumes {
-		if storagetypes.IsHotplugVolume(&v) {
-			hotplugVols[v.Name] = true
-		}
-	}
 
 	for i, d := range dom.Devices.Disks {
 		if d.Alias == nil {
@@ -890,39 +906,30 @@ func configureLocalDiskToMigrate(dom *libvirtxml.Domain, vmi *v1.VirtualMachineI
 		// Configure the slice to enable to migrate the volume to a destination with different size
 		// See suggestion in: https://issues.redhat.com/browse/RHEL-4607
 		if dom.Devices.Disks[i].Source.Slices == nil {
-			dom.Devices.Disks[i].Source.Slices = &libvirtxml.DomainDiskSlices{
-				Slices: []libvirtxml.DomainDiskSlice{
-					{
-						Type:   "storage",
-						Offset: 0,
-						Size:   uint(size),
-					},
-				}}
+			dom.Devices.Disks[i].Source.Slices = &libvirtxml.DomainDiskSlices{}
+		}
+		slice := libvirtxml.DomainDiskSlice{
+			Type:   "storage",
+			Offset: 0,
+			Size:   uint(size),
+		}
+		if len(dom.Devices.Disks[i].Source.Slices.Slices) > 0 {
+			dom.Devices.Disks[i].Source.Slices.Slices[0] = slice
+		} else {
+			dom.Devices.Disks[i].Source.Slices.Slices = append(dom.Devices.Disks[i].Source.Slices.Slices, slice)
 		}
-		var path string
-		_, hotplugVol := hotplugVols[name]
 		// Adjust the XML configuration when it migrates from a filesystem source to a block destination or vice versa
 		if _, ok := fsSrcBlockDstVols[name]; ok {
 			log.Log.V(2).Infof("Replace filesystem source with block destination for volume %s", name)
-			if hotplugVol {
-				path = hotplugdisk.GetVolumeMountDir(name)
-			} else {
-				path = filepath.Join(string(filepath.Separator), "dev", name)
-			}
 			dom.Devices.Disks[i].Source.Block = &libvirtxml.DomainDiskSourceBlock{
-				Dev: path,
+				Dev: filepath.Join(string(filepath.Separator), "dev", name),
 			}
 			dom.Devices.Disks[i].Source.File = nil
 		}
 		if _, ok := blockSrcFsDstVols[name]; ok {
 			log.Log.V(2).Infof("Replace block source with destination for volume %s", name)
-			if hotplugVol {
-				path = hotplugdisk.GetVolumeMountDir(name) + ".img"
-			} else {
-				path = filepath.Join(hostdisk.GetMountedHostDiskDir(name), "disk.img")
-			}
 			dom.Devices.Disks[i].Source.File = &libvirtxml.DomainDiskSourceFile{
-				File: path,
+				File: filepath.Join(hostdisk.GetMountedHostDiskDir(name), "disk.img"),
 			}
 			dom.Devices.Disks[i].Source.Block = nil
 		}
@@ -1042,3 +1049,19 @@ func (l *LibvirtDomainManager) updateVMIMigrationMode(mode v1.MigrationMode) {
 	})
 	log.Log.V(4).Infof("Migration mode set in metadata: %s", l.metadataCache.Migration.String())
 }
+
+func shouldConfigureParallelMigration(options *cmdclient.MigrationOptions) (shouldConfigure bool, threadsCount int) {
+	if options == nil {
+		return
+	}
+	if options.AllowPostCopy {
+		return
+	}
+	if options.ParallelMigrationThreads == nil {
+		return
+	}
+
+	shouldConfigure = true
+	threadsCount = int(*options.ParallelMigrationThreads)
+	return
+}
