@@ -30,11 +30,12 @@ import (
 type IOThreadsPolicy string
 
 const (
-	IOThreadsPolicyShared  IOThreadsPolicy = "shared"
-	IOThreadsPolicyAuto    IOThreadsPolicy = "auto"
-	CPUModeHostPassthrough                 = "host-passthrough"
-	CPUModeHostModel                       = "host-model"
-	DefaultCPUModel                        = CPUModeHostModel
+	IOThreadsPolicyShared           IOThreadsPolicy = "shared"
+	IOThreadsPolicyAuto             IOThreadsPolicy = "auto"
+	IOThreadsPolicySupplementalPool IOThreadsPolicy = "supplementalPool"
+	CPUModeHostPassthrough                          = "host-passthrough"
+	CPUModeHostModel                                = "host-model"
+	DefaultCPUModel                                 = CPUModeHostModel
 )
 
 const HotplugDiskDir = "/var/run/kubevirt/hotplug-disks/"
@@ -203,6 +204,9 @@ type DomainSpec struct {
 	// One of: shared, auto
 	// +optional
 	IOThreadsPolicy *IOThreadsPolicy `json:"ioThreadsPolicy,omitempty"`
+	// IOThreads specifies the IOThreads options.
+	// +optional
+	IOThreads *DiskIOThreads `json:"ioThreads,omitempty"`
 	// Chassis specifies the chassis info passed to the domain.
 	// +optional
 	Chassis *Chassis `json:"chassis,omitempty"`
@@ -543,9 +547,6 @@ type SoundDevice struct {
 }
 
 type TPMDevice struct {
-	// Enabled allows a user to explictly disable the vTPM even when one is enabled by a preference referenced by the VirtualMachine
-	// Defaults to True
-	Enabled *bool `json:"enabled,omitempty"`
 	// Persistent indicates the state of the TPM device should be kept accross reboots
 	// Defaults to false
 	Persistent *bool `json:"persistent,omitempty"`
@@ -861,7 +862,6 @@ type HotplugVolumeSource struct {
 
 type DataVolumeSource struct {
 	// Name of both the DataVolume and the PVC in the same namespace.
-	// After PVC population the DataVolume is garbage collected by default.
 	Name string `json:"name"`
 	// Hotpluggable indicates whether the volume can be hotplugged and hotunplugged.
 	// +optional
@@ -1280,7 +1280,11 @@ type Interface struct {
 	// +optional
 	ACPIIndex int `json:"acpiIndex,omitempty"`
 	// State represents the requested operational state of the interface.
-	// The (only) value supported is `absent`, expressing a request to remove the interface.
+	// The supported values are:
+	// `absent`, expressing a request to remove the interface.
+	// `down`, expressing a request to set the link down.
+	// `up`, expressing a request to set the link up.
+	// Empty value functions as `up`.
 	// +optional
 	State InterfaceState `json:"state,omitempty"`
 }
@@ -1288,7 +1292,9 @@ type Interface struct {
 type InterfaceState string
 
 const (
-	InterfaceStateAbsent InterfaceState = "absent"
+	InterfaceStateAbsent   InterfaceState = "absent"
+	InterfaceStateLinkUp   InterfaceState = "up"
+	InterfaceStateLinkDown InterfaceState = "down"
 )
 
 // Extra DHCP options to use in the interface.
@@ -1589,3 +1595,9 @@ type CPUTopology struct {
 	// Must be a value greater or equal 1.
 	Threads uint32 `json:"threads,omitempty"`
 }
+
+type DiskIOThreads struct {
+	// SupplementalPoolThreadCount specifies how many iothreads are allocated for the supplementalPool policy.
+	// +optional
+	SupplementalPoolThreadCount *uint32 `json:"supplementalPoolThreadCount,omitempty"`
+}
