@@ -7,8 +7,6 @@ import (
 	"time"
 
 	"kubevirt.io/kubevirt/tests/decorators"
-	"kubevirt.io/kubevirt/tests/libkubevirt"
-	"kubevirt.io/kubevirt/tests/libkubevirt/config"
 	"kubevirt.io/kubevirt/tests/libpod"
 	"kubevirt.io/kubevirt/tests/libvmops"
 	"kubevirt.io/kubevirt/tests/watcher"
@@ -27,6 +25,7 @@ import (
 	"k8s.io/apimachinery/pkg/util/rand"
 
 	"kubevirt.io/kubevirt/pkg/apimachinery/patch"
+	"kubevirt.io/kubevirt/pkg/libdv"
 	"kubevirt.io/kubevirt/pkg/libvmi"
 	libvmici "kubevirt.io/kubevirt/pkg/libvmi/cloudinit"
 	"kubevirt.io/kubevirt/pkg/pointer"
@@ -47,7 +46,6 @@ import (
 
 	"kubevirt.io/kubevirt/tests/console"
 	cd "kubevirt.io/kubevirt/tests/containerdisk"
-	"kubevirt.io/kubevirt/tests/libdv"
 	"kubevirt.io/kubevirt/tests/libnet"
 	"kubevirt.io/kubevirt/tests/libstorage"
 	"kubevirt.io/kubevirt/tests/libvmifact"
@@ -73,16 +71,6 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 		webhook    *admissionregistrationv1.ValidatingWebhookConfiguration
 	)
 
-	waitSnapshotReady := func() {
-		Eventually(func() *snapshotv1.VirtualMachineSnapshotStatus {
-			snapshot, err = virtClient.VirtualMachineSnapshot(vm.Namespace).Get(context.Background(), snapshot.Name, metav1.GetOptions{})
-			Expect(err).ToNot(HaveOccurred())
-			return snapshot.Status
-		}, 180*time.Second, time.Second).Should(gstruct.PointTo(gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
-			"ReadyToUse": gstruct.PointTo(BeTrue()),
-		})))
-	}
-
 	deleteSnapshot := func() {
 		err := virtClient.VirtualMachineSnapshot(snapshot.Namespace).Delete(context.Background(), snapshot.Name, metav1.DeleteOptions{})
 		if errors.IsNotFound(err) {
@@ -120,51 +108,6 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 		}, 180*time.Second, time.Second).ShouldNot(BeEmpty())
 	}
 
-	createDenyVolumeSnapshotCreateWebhook := func() {
-		fp := admissionregistrationv1.Fail
-		sideEffectNone := admissionregistrationv1.SideEffectClassNone
-		whPath := "/foobar"
-		whName := "dummy-webhook-deny-volume-snapshot-create.kubevirt.io"
-		wh := &admissionregistrationv1.ValidatingWebhookConfiguration{
-			ObjectMeta: metav1.ObjectMeta{
-				Name: "temp-webhook-deny-volume-snapshot-create-" + rand.String(5),
-			},
-			Webhooks: []admissionregistrationv1.ValidatingWebhook{
-				{
-					Name:                    whName,
-					AdmissionReviewVersions: []string{"v1", "v1beta1"},
-					FailurePolicy:           &fp,
-					SideEffects:             &sideEffectNone,
-					Rules: []admissionregistrationv1.RuleWithOperations{{
-						Operations: []admissionregistrationv1.OperationType{
-							admissionregistrationv1.Create,
-						},
-						Rule: admissionregistrationv1.Rule{
-							APIGroups:   []string{vsv1.GroupName},
-							APIVersions: []string{vsv1.SchemeGroupVersion.Version},
-							Resources:   []string{"volumesnapshots"},
-						},
-					}},
-					ClientConfig: admissionregistrationv1.WebhookClientConfig{
-						Service: &admissionregistrationv1.ServiceReference{
-							Namespace: testsuite.GetTestNamespace(nil),
-							Name:      "nonexistant",
-							Path:      &whPath,
-						},
-					},
-					ObjectSelector: &metav1.LabelSelector{
-						MatchLabels: map[string]string{
-							"snapshot.kubevirt.io/source-vm-name": vm.Name,
-						},
-					},
-				},
-			},
-		}
-		wh, err := virtClient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Create(context.Background(), wh, metav1.CreateOptions{})
-		Expect(err).ToNot(HaveOccurred())
-		webhook = wh
-	}
-
 	checkOnlineSnapshotExpectedContentSource := func(vm *v1.VirtualMachine, contentName string, expectVolumeBackups bool) {
 		content, err := virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
@@ -209,7 +152,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 			_, err := virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 			Expect(err).ToNot(HaveOccurred())
 
-			waitSnapshotReady()
+			snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 
 			Expect(snapshot.Status.SourceUID).ToNot(BeNil())
 			Expect(*snapshot.Status.SourceUID).To(Equal(vm.UID))
@@ -225,7 +168,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 
 			contentName := *snapshot.Status.VirtualMachineSnapshotContentName
 			if running {
-				expectedIndications := []snapshotv1.Indication{snapshotv1.VMSnapshotNoGuestAgentIndication, snapshotv1.VMSnapshotOnlineSnapshotIndication}
+				expectedIndications := []snapshotv1.Indication{snapshotv1.VMSnapshotOnlineSnapshotIndication, snapshotv1.VMSnapshotNoGuestAgentIndication}
 				Expect(snapshot.Status.Indications).To(Equal(expectedIndications))
 				checkOnlineSnapshotExpectedContentSource(vm, contentName, false)
 			} else {
@@ -240,11 +183,11 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 			}
 		}
 
-		It("[test_id:4609]should successfully create a snapshot", func() {
+		It("[test_id:4609]should successfully create a snapshot", decorators.StorageCritical, func() {
 			createAndVerifyVMSnapshot(vm)
 		})
 
-		It("[test_id:4610]create a snapshot when VM is running should succeed", func() {
+		It("[test_id:4610]create a snapshot when VM is running should succeed", decorators.StorageCritical, func() {
 			patch, err := patch.New(patch.WithReplace("/spec/runStrategy", v1.RunStrategyAlways)).GeneratePayload()
 			Expect(err).ToNot(HaveOccurred())
 
@@ -291,7 +234,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 		})
 	})
 
-	Context("[storage-req]", decorators.StorageReq, func() {
+	Context("[storage-req]", decorators.StorageReq, decorators.RequiresSnapshotStorageClass, func() {
 		var (
 			snapshotStorageClass string
 		)
@@ -301,7 +244,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 			Expect(err).ToNot(HaveOccurred())
 
 			if sc == "" {
-				Skip("Skiping test, no VolumeSnapshot support")
+				Fail("Failing test, no VolumeSnapshot support")
 			}
 
 			snapshotStorageClass = sc
@@ -323,9 +266,9 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 			checkVMFreeze := func(snapshot *snapshotv1.VirtualMachineSnapshot, vmi *v1.VirtualMachineInstance, hasGuestAgent bool, assertionFunc func(*v1.VirtualMachineInstance)) {
 				var expectedIndications []snapshotv1.Indication
 				if hasGuestAgent {
-					expectedIndications = []snapshotv1.Indication{snapshotv1.VMSnapshotGuestAgentIndication, snapshotv1.VMSnapshotOnlineSnapshotIndication}
+					expectedIndications = []snapshotv1.Indication{snapshotv1.VMSnapshotOnlineSnapshotIndication, snapshotv1.VMSnapshotGuestAgentIndication}
 				} else {
-					expectedIndications = []snapshotv1.Indication{snapshotv1.VMSnapshotNoGuestAgentIndication, snapshotv1.VMSnapshotOnlineSnapshotIndication}
+					expectedIndications = []snapshotv1.Indication{snapshotv1.VMSnapshotOnlineSnapshotIndication, snapshotv1.VMSnapshotNoGuestAgentIndication}
 				}
 				Expect(snapshot.Status.Indications).To(Equal(expectedIndications))
 
@@ -407,7 +350,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				}, 30)).To(Succeed())
 			}
 
-			It("[test_id:6767]with volumes and guest agent available", func() {
+			It("[test_id:6767]with volumes and guest agent available", decorators.StorageCritical, func() {
 				quantity, err := resource.ParseQuantity("1Gi")
 				Expect(err).ToNot(HaveOccurred())
 				vmi := libvmifact.NewFedora(libnet.WithMasqueradeNetworking())
@@ -468,7 +411,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 				checkVMFreeze(snapshot, vmi, true, ensureFreezeFedora)
 
 				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
@@ -476,7 +419,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				checkContentSourceAndMemory(vm.DeepCopy(), contentName, initialMemory)
 			})
 
-			It("[test_id:6768]with volumes and no guest agent available", func() {
+			It("[test_id:6768]with volumes and no guest agent available", decorators.StorageCritical, func() {
 				quantity, err := resource.ParseQuantity("1Gi")
 				Expect(err).ToNot(HaveOccurred())
 				vmi := libvmifact.NewAlpine(libnet.WithMasqueradeNetworking())
@@ -519,7 +462,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 				checkVMFreeze(snapshot, vmi, false, ensureNoFreezeAlpine)
 
 				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
@@ -543,7 +486,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 				checkVMFreeze(snapshot, vmi, true, ensureNoFreezeAlpine)
 
 				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
@@ -558,7 +501,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				Eventually(matcher.ThisVMI(vmi), 12*time.Minute, 2*time.Second).Should(matcher.HaveConditionTrue(v1.VirtualMachineInstanceAgentConnected))
 				Expect(console.LoginToFedora(vmi)).To(Succeed())
 
-				createDenyVolumeSnapshotCreateWebhook()
+				webhook = createDenyVolumeSnapshotCreateWebhook(virtClient, vm.Name)
 				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
 
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
@@ -585,7 +528,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				Eventually(matcher.ThisVMI(vmi), 12*time.Minute, 2*time.Second).Should(matcher.HaveConditionTrue(v1.VirtualMachineInstanceAgentConnected))
 				Expect(console.LoginToFedora(vmi)).To(Succeed())
 
-				createDenyVolumeSnapshotCreateWebhook()
+				webhook = createDenyVolumeSnapshotCreateWebhook(virtClient, vm.Name)
 				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
 				snapshot.Spec.FailureDeadline = &metav1.Duration{Duration: 40 * time.Second}
 
@@ -612,7 +555,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
 							"Type":   Equal(snapshotv1.ConditionProgressing),
 							"Status": Equal(corev1.ConditionFalse),
-							"Reason": Equal(snapshotDeadlineExceeded)}),
+							"Reason": Equal("Operation failed")}),
 						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
 							"Type":   Equal(snapshotv1.ConditionReady),
 							"Status": Equal(corev1.ConditionFalse),
@@ -653,7 +596,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				Expect(err).ToNot(HaveOccurred())
 
 				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
-				expectedIndications := []snapshotv1.Indication{snapshotv1.VMSnapshotGuestAgentIndication, snapshotv1.VMSnapshotOnlineSnapshotIndication}
+				expectedIndications := []snapshotv1.Indication{snapshotv1.VMSnapshotOnlineSnapshotIndication, snapshotv1.VMSnapshotGuestAgentIndication}
 				Expect(snapshot.Status.Indications).To(Equal(expectedIndications))
 
 				updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(context.Background(), vm.Name, metav1.GetOptions{})
@@ -901,9 +844,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 					memoryDumpPVC.Spec.VolumeMode = &volumeMode
 					var err error
 					memoryDumpPVC, err = virtClient.CoreV1().PersistentVolumeClaims(testsuite.GetTestNamespace(nil)).Create(context.Background(), memoryDumpPVC, metav1.CreateOptions{})
-					if err != nil {
-						Skip(fmt.Sprintf("Skiping test, no filesystem pvc available, err: %s", err))
-					}
+					Expect(err).ToNot(HaveOccurred())
 				})
 
 				AfterEach(func() {
@@ -1022,13 +963,61 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				}
 			})
 
+			It("[test_id:4611] should successfully create a snapshot", decorators.StorageCritical, func() {
+				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
+
+				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
+
+				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
+				contentName := *snapshot.Status.VirtualMachineSnapshotContentName
+				content, err := virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				Expect(*content.Spec.VirtualMachineSnapshotName).To(Equal(snapshot.Name))
+				Expect(content.Spec.Source.VirtualMachine.Spec).To(Equal(vm.Spec))
+				Expect(content.Spec.VolumeBackups).Should(HaveLen(len(vm.Spec.DataVolumeTemplates)))
+
+				for _, vol := range vm.Spec.Template.Spec.Volumes {
+					if vol.DataVolume == nil {
+						continue
+					}
+					found := false
+					for _, vb := range content.Spec.VolumeBackups {
+						if vol.DataVolume.Name == vb.PersistentVolumeClaim.Name {
+							found = true
+							Expect(vol.Name).To(Equal(vb.VolumeName))
+
+							pvc, err := virtClient.CoreV1().PersistentVolumeClaims(vm.Namespace).Get(context.Background(), vol.DataVolume.Name, metav1.GetOptions{})
+							Expect(err).ToNot(HaveOccurred())
+							Expect(pvc.Spec).To(Equal(vb.PersistentVolumeClaim.Spec))
+
+							Expect(vb.VolumeSnapshotName).ToNot(BeNil())
+							vs, err := virtClient.
+								KubernetesSnapshotClient().
+								SnapshotV1().
+								VolumeSnapshots(vm.Namespace).
+								Get(context.Background(), *vb.VolumeSnapshotName, metav1.GetOptions{})
+							Expect(err).ToNot(HaveOccurred())
+							Expect(*vs.Spec.Source.PersistentVolumeClaimName).Should(Equal(vol.DataVolume.Name))
+							Expect(vs.Labels["snapshot.kubevirt.io/source-vm-name"]).Should(Equal(vm.Name))
+							Expect(vs.Status.Error).To(BeNil())
+							Expect(*vs.Status.ReadyToUse).To(BeTrue())
+						}
+					}
+					Expect(found).To(BeTrue())
+				}
+			})
+
 			It("should successfully recreate status", func() {
 				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
 
 				ss, err := virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 
 				ss, err = virtClient.VirtualMachineSnapshot(ss.Namespace).Get(context.Background(), ss.Name, metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
@@ -1088,7 +1077,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 
 				cn := snapshot.Status.VirtualMachineSnapshotContentName
 				Expect(cn).ToNot(BeNil())
@@ -1121,7 +1110,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 
 				cn := snapshot.Status.VirtualMachineSnapshotContentName
 				Expect(cn).ToNot(BeNil())
@@ -1178,63 +1167,8 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				Expect(*snapshot.Status.ReadyToUse).To(BeTrue())
 			})
 
-			It("[test_id:6952]snapshot change phase to in progress and succeeded and then should not fail", func() {
-				createDenyVolumeSnapshotCreateWebhook()
-				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
-				snapshot.Spec.FailureDeadline = &metav1.Duration{Duration: time.Minute}
-
-				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
-				Expect(err).ToNot(HaveOccurred())
-
-				Eventually(func() *snapshotv1.VirtualMachineSnapshotStatus {
-					snapshot, err = virtClient.VirtualMachineSnapshot(vm.Namespace).Get(context.Background(), snapshot.Name, metav1.GetOptions{})
-					Expect(err).ToNot(HaveOccurred())
-
-					return snapshot.Status
-				}, 30*time.Second, 2*time.Second).Should(gstruct.PointTo(gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
-					"Conditions": ContainElements(
-						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
-							"Type":   Equal(snapshotv1.ConditionReady),
-							"Status": Equal(corev1.ConditionFalse)}),
-						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
-							"Type":   Equal(snapshotv1.ConditionProgressing),
-							"Status": Equal(corev1.ConditionTrue),
-							"Reason": Equal("Source locked and operation in progress")}),
-					),
-					"Phase": Equal(snapshotv1.InProgress),
-				})))
-
-				updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(context.Background(), vm.Name, metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-				Expect(*updatedVM.Status.SnapshotInProgress).To(Equal(snapshot.Name))
-
-				Expect(snapshot.Status.CreationTime).To(BeNil())
-
-				contentName := fmt.Sprintf("%s-%s", vmSnapshotContent, snapshot.UID)
-				content, err := virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-				Expect(content.Status).To(BeNil())
-
-				deleteWebhook()
-
-				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
-
-				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
-				content, err = virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-
-				Expect(*content.Spec.VirtualMachineSnapshotName).To(Equal(snapshot.Name))
-				Expect(content.Spec.Source.VirtualMachine.Spec).To(Equal(vm.Spec))
-				Expect(content.Spec.VolumeBackups).Should(HaveLen(len(vm.Spec.DataVolumeTemplates)))
-
-				// Sleep to pass the time of the deadline
-				time.Sleep(time.Second)
-				// If snapshot succeeded it should not change to failure when deadline exceeded
-				Expect(snapshot.Status.Phase).To(Equal(snapshotv1.Succeeded))
-			})
-
 			It("[test_id:6838]snapshot should fail when deadline exceeded due to volume snapshots failure", func() {
-				createDenyVolumeSnapshotCreateWebhook()
+				webhook = createDenyVolumeSnapshotCreateWebhook(virtClient, vm.Name)
 				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
 				snapshot.Spec.FailureDeadline = &metav1.Duration{Duration: 40 * time.Second}
 
@@ -1259,7 +1193,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
 							"Type":   Equal(snapshotv1.ConditionProgressing),
 							"Status": Equal(corev1.ConditionFalse),
-							"Reason": Equal(snapshotDeadlineExceeded)}),
+							"Reason": Equal("Operation failed")}),
 						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
 							"Type":   Equal(snapshotv1.ConditionFailure),
 							"Status": Equal(corev1.ConditionTrue),
@@ -1333,101 +1267,145 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 			})))
 		})
 
-		Context("[Serial]With more complicated VM with/out GC of succeeded DV", Serial, func() {
-			var originalTTL *int32
-
-			BeforeEach(func() {
-				cdi := libstorage.GetCDI(virtClient)
-				originalTTL = cdi.Spec.Config.DataVolumeTTLSeconds
-			})
-
-			AfterEach(func() {
-				libstorage.SetDataVolumeGC(virtClient, originalTTL)
-			})
-
-			DescribeTable("should successfully create a snapshot", func(ttl *int32) {
-				libstorage.SetDataVolumeGC(virtClient, ttl)
-
-				vm = renderVMWithRegistryImportDataVolume(cd.ContainerDiskAlpine, snapshotStorageClass)
-				wffcSC := libstorage.IsStorageClassBindingModeWaitForFirstConsumer(snapshotStorageClass)
-				if wffcSC {
-					// with wffc need to start the virtual machine
-					// in order for the pvc to be populated
-					vm.Spec.RunStrategy = virtpointer.P(v1.RunStrategyAlways)
-				} else {
-					vm.Spec.RunStrategy = virtpointer.P(v1.RunStrategyHalted)
-				}
-
-				vm, err = virtClient.VirtualMachine(vm.Namespace).Create(context.Background(), vm, metav1.CreateOptions{})
-				Expect(err).ToNot(HaveOccurred())
-
-				for _, dvt := range vm.Spec.DataVolumeTemplates {
-					waitDataVolumePopulated(vm.Namespace, dvt.Name)
-				}
-
-				if wffcSC {
-					vm = libvmops.StopVirtualMachine(vm)
-				}
-
-				snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
-
-				_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
-				Expect(err).ToNot(HaveOccurred())
-
-				waitSnapshotReady()
-
-				Expect(snapshot.Status.CreationTime).ToNot(BeNil())
-				contentName := *snapshot.Status.VirtualMachineSnapshotContentName
-				content, err := virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
-				Expect(err).ToNot(HaveOccurred())
-
-				Expect(*content.Spec.VirtualMachineSnapshotName).To(Equal(snapshot.Name))
-				Expect(content.Spec.Source.VirtualMachine.Spec).To(Equal(vm.Spec))
-				Expect(content.Spec.VolumeBackups).Should(HaveLen(len(vm.Spec.DataVolumeTemplates)))
-
-				for _, vol := range vm.Spec.Template.Spec.Volumes {
-					if vol.DataVolume == nil {
-						continue
-					}
-					found := false
-					for _, vb := range content.Spec.VolumeBackups {
-						if vol.DataVolume.Name == vb.PersistentVolumeClaim.Name {
-							found = true
-							Expect(vol.Name).To(Equal(vb.VolumeName))
-
-							pvc, err := virtClient.CoreV1().PersistentVolumeClaims(vm.Namespace).Get(context.Background(), vol.DataVolume.Name, metav1.GetOptions{})
-							Expect(err).ToNot(HaveOccurred())
-							Expect(pvc.Spec).To(Equal(vb.PersistentVolumeClaim.Spec))
-
-							Expect(vb.VolumeSnapshotName).ToNot(BeNil())
-							vs, err := virtClient.
-								KubernetesSnapshotClient().
-								SnapshotV1().
-								VolumeSnapshots(vm.Namespace).
-								Get(context.Background(), *vb.VolumeSnapshotName, metav1.GetOptions{})
-							Expect(err).ToNot(HaveOccurred())
-							Expect(*vs.Spec.Source.PersistentVolumeClaimName).Should(Equal(vol.DataVolume.Name))
-							Expect(vs.Labels["snapshot.kubevirt.io/source-vm-name"]).Should(Equal(vm.Name))
-							Expect(vs.Status.Error).To(BeNil())
-							Expect(*vs.Status.ReadyToUse).To(BeTrue())
-						}
-					}
-					Expect(found).To(BeTrue())
-				}
-			},
-				Entry("[test_id:4611] without DV garbage collection", virtpointer.P(int32(-1))),
-				Entry("[test_id:8668] with DV garbage collection", virtpointer.P(int32(0))),
+		It("snapshot create before source wait for volume bound, then continues, succeeds and then should not fail", func() {
+			wffc := libstorage.IsStorageClassBindingModeWaitForFirstConsumer(snapshotStorageClass)
+			// Stand alone dv
+			dv := libdv.NewDataVolume(
+				libdv.WithBlankImageSource(),
+				libdv.WithStorage(
+					libdv.StorageWithStorageClass(snapshotStorageClass),
+					libdv.StorageWithVolumeSize(cd.BlankVolumeSize)),
 			)
+			dv, err = virtClient.CdiClient().CdiV1beta1().DataVolumes(testsuite.GetTestNamespace(nil)).Create(context.Background(), dv, metav1.CreateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			// DV for datavolumetemplate
+			dv2 := libdv.NewDataVolume(
+				libdv.WithNamespace(testsuite.GetTestNamespace(nil)),
+				libdv.WithRegistryURLSource(cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskAlpine)),
+				libdv.WithStorage(
+					libdv.StorageWithStorageClass(snapshotStorageClass),
+					libdv.StorageWithVolumeSize(cd.ContainerDiskSizeBySourceURL(cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskAlpine))),
+				),
+			)
+			vm := libvmi.NewVirtualMachine(
+				libvmi.New(
+					libvmi.WithNamespace(testsuite.GetTestNamespace(nil)),
+					libvmi.WithDataVolume("disk0", dv.Name),
+					libvmi.WithDataVolume("disk1", dv2.Name),
+					libvmi.WithResourceMemory("128Mi"),
+				),
+				libvmi.WithDataVolumeTemplate(dv2),
+				libvmi.WithRunStrategy(v1.RunStrategyHalted),
+			)
+
+			By("Create Snapshot before source VM")
+			webhook = createDenyVolumeSnapshotCreateWebhook(virtClient, vm.Name)
+			snapshot = libstorage.NewSnapshot(vm.Name, vm.Namespace)
+			snapshot.Spec.FailureDeadline = &metav1.Duration{Duration: time.Minute}
+
+			_, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			By("Wait Snapshot conditions to reflect source doesnt exist")
+			Eventually(func() *snapshotv1.VirtualMachineSnapshotStatus {
+				snapshot, err = virtClient.VirtualMachineSnapshot(vm.Namespace).Get(context.Background(), snapshot.Name, metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				return snapshot.Status
+			}, 30*time.Second, 2*time.Second).Should(gstruct.PointTo(gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+				"Conditions": ContainElements(
+					gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+						"Type":   Equal(snapshotv1.ConditionProgressing),
+						"Status": Equal(corev1.ConditionFalse),
+						"Reason": Equal("Source does not exist")}),
+					gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+						"Type":   Equal(snapshotv1.ConditionReady),
+						"Status": Equal(corev1.ConditionFalse)}),
+				),
+				"Phase": Equal(snapshotv1.InProgress),
+			})))
+
+			By("Create offline VM")
+			vm, err := virtClient.VirtualMachine(vm.Namespace).Create(context.Background(), vm, metav1.CreateOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			if wffc {
+				By("Wait Snapshot conditions to reflect wait for volume bound")
+				Eventually(func() *snapshotv1.VirtualMachineSnapshotStatus {
+					snapshot, err = virtClient.VirtualMachineSnapshot(vm.Namespace).Get(context.Background(), snapshot.Name, metav1.GetOptions{})
+					Expect(err).ToNot(HaveOccurred())
+
+					return snapshot.Status
+				}, 30*time.Second, 2*time.Second).Should(gstruct.PointTo(gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+					"Conditions": ContainElements(
+						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+							"Type":   Equal(snapshotv1.ConditionProgressing),
+							"Status": Equal(corev1.ConditionFalse),
+							"Reason": ContainSubstring(fmt.Sprintf("Source not locked source %s/%s volume not bound", vm.Namespace, vm.Name))}),
+						gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+							"Type":   Equal(snapshotv1.ConditionReady),
+							"Status": Equal(corev1.ConditionFalse)}),
+					),
+					"Phase": Equal(snapshotv1.InProgress),
+				})))
+
+				By("Starting the VM which should bind the volumes")
+				vm = libvmops.StartVirtualMachine(vm)
+			}
+
+			Eventually(func() *snapshotv1.VirtualMachineSnapshotStatus {
+				snapshot, err = virtClient.VirtualMachineSnapshot(vm.Namespace).Get(context.Background(), snapshot.Name, metav1.GetOptions{})
+				Expect(err).ToNot(HaveOccurred())
+
+				return snapshot.Status
+			}, 120*time.Second, 2*time.Second).Should(gstruct.PointTo(gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+				"Conditions": ContainElements(
+					gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+						"Type":   Equal(snapshotv1.ConditionReady),
+						"Status": Equal(corev1.ConditionFalse)}),
+					gstruct.MatchFields(gstruct.IgnoreExtras, gstruct.Fields{
+						"Type":   Equal(snapshotv1.ConditionProgressing),
+						"Status": Equal(corev1.ConditionTrue),
+						"Reason": Equal("Source locked and operation in progress")}),
+				),
+				"Phase": Equal(snapshotv1.InProgress),
+			})))
+
+			updatedVM, err := virtClient.VirtualMachine(vm.Namespace).Get(context.Background(), vm.Name, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(*updatedVM.Status.SnapshotInProgress).To(Equal(snapshot.Name))
+
+			Expect(snapshot.Status.CreationTime).To(BeNil())
+
+			contentName := fmt.Sprintf("%s-%s", vmSnapshotContent, snapshot.UID)
+			content, err := virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+			Expect(content.Status).To(BeNil())
+
+			deleteWebhook()
+
+			snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
+
+			Expect(snapshot.Status.CreationTime).ToNot(BeNil())
+			content, err = virtClient.VirtualMachineSnapshotContent(vm.Namespace).Get(context.Background(), contentName, metav1.GetOptions{})
+			Expect(err).ToNot(HaveOccurred())
+
+			Expect(*content.Spec.VirtualMachineSnapshotName).To(Equal(snapshot.Name))
+			Expect(content.Spec.Source.VirtualMachine.Spec).To(Equal(vm.Spec))
+			Expect(content.Spec.VolumeBackups).Should(HaveLen(len(vm.Spec.Template.Spec.Volumes)))
+
+			// Sleep to pass the time of the deadline
+			time.Sleep(time.Second)
+			// If snapshot succeeded it should not change to failure when deadline exceeded
+			Expect(snapshot.Status.Phase).To(Equal(snapshotv1.Succeeded))
+			Expect(snapshot.Status.SnapshotVolumes.IncludedVolumes).Should(HaveLen(2))
 		})
 
 		Context("with independent DataVolume", func() {
 			var dv *cdiv1.DataVolume
 
-			AfterEach(func() {
-				libstorage.DeleteDataVolume(&dv)
-			})
-
-			DescribeTable("should accurately report DataVolume provisioning", func(storageOptFun func(string, string) libvmi.Option, memory string) {
+			DescribeTable("should accurately report DataVolume provisioning", func(storageOptFun func(string, string, ...libvmi.DiskOption) libvmi.Option, memory string) {
 				dataVolume := libdv.NewDataVolume(
 					libdv.WithRegistryURLSourceAndPullMethod(cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskAlpine), cdiv1.RegistryPullNode),
 					libdv.WithStorage(libdv.StorageWithStorageClass(snapshotStorageClass)),
@@ -1535,12 +1513,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				Expect(snapshot.Status.SnapshotVolumes.ExcludedVolumes[0]).Should(Equal("notsnapshotablevolume"))
 			})
 
-			It("Should also include backend PVC in the snapshot", Serial, func() {
-				By("Setting the VMState storage class to snapshot")
-				kv := libkubevirt.GetCurrentKv(virtClient)
-				kv.Spec.Configuration.VMStateStorageClass = snapshotStorageClass
-				config.UpdateKubeVirtConfigValueAndWait(kv.Spec.Configuration)
-
+			It("Should also include backend PVC in the snapshot", func() {
 				By("Creating DV with snapshot supported storage class")
 				includedDataVolume := libdv.NewDataVolume(
 					libdv.WithRegistryURLSourceAndPullMethod(cd.DataVolumeImportUrlForContainerDisk(cd.ContainerDiskAlpine), cdiv1.RegistryPullNode),
@@ -1636,7 +1609,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				}
 			})
 
-			DescribeTable("Bug #8435 - should create a snapshot successfully", func(toRunSourceVM bool) {
+			DescribeTable("Bug #8435 - should create a snapshot successfully", decorators.StorageCritical, func(toRunSourceVM bool) {
 				if !toRunSourceVM {
 					By("Stoping the VM")
 					vm = libvmops.StopVirtualMachine(vm)
@@ -1646,7 +1619,7 @@ var _ = SIGDescribe("VirtualMachineSnapshot Tests", func() {
 				snapshot, err = virtClient.VirtualMachineSnapshot(snapshot.Namespace).Create(context.Background(), snapshot, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 
-				waitSnapshotReady()
+				snapshot = libstorage.WaitSnapshotSucceeded(virtClient, vm.Namespace, snapshot.Name)
 			},
 				Entry("with running source VM", true),
 				Entry("with stopped source VM", false),
@@ -1710,3 +1683,48 @@ func clearConditionsTimestamps(conditions []snapshotv1.Condition) {
 		conditions[i].LastTransitionTime = emptyTime
 	}
 }
+
+func createDenyVolumeSnapshotCreateWebhook(virtClient kubecli.KubevirtClient, vmName string) *admissionregistrationv1.ValidatingWebhookConfiguration {
+	fp := admissionregistrationv1.Fail
+	sideEffectNone := admissionregistrationv1.SideEffectClassNone
+	whPath := "/foobar"
+	whName := "dummy-webhook-deny-volume-snapshot-create.kubevirt.io"
+	wh := &admissionregistrationv1.ValidatingWebhookConfiguration{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "temp-webhook-deny-volume-snapshot-create-" + rand.String(5),
+		},
+		Webhooks: []admissionregistrationv1.ValidatingWebhook{
+			{
+				Name:                    whName,
+				AdmissionReviewVersions: []string{"v1", "v1beta1"},
+				FailurePolicy:           &fp,
+				SideEffects:             &sideEffectNone,
+				Rules: []admissionregistrationv1.RuleWithOperations{{
+					Operations: []admissionregistrationv1.OperationType{
+						admissionregistrationv1.Create,
+					},
+					Rule: admissionregistrationv1.Rule{
+						APIGroups:   []string{vsv1.GroupName},
+						APIVersions: []string{vsv1.SchemeGroupVersion.Version},
+						Resources:   []string{"volumesnapshots"},
+					},
+				}},
+				ClientConfig: admissionregistrationv1.WebhookClientConfig{
+					Service: &admissionregistrationv1.ServiceReference{
+						Namespace: testsuite.GetTestNamespace(nil),
+						Name:      "nonexistant",
+						Path:      &whPath,
+					},
+				},
+				ObjectSelector: &metav1.LabelSelector{
+					MatchLabels: map[string]string{
+						"snapshot.kubevirt.io/source-vm-name": vmName,
+					},
+				},
+			},
+		},
+	}
+	wh, err := virtClient.AdmissionregistrationV1().ValidatingWebhookConfigurations().Create(context.Background(), wh, metav1.CreateOptions{})
+	Expect(err).ToNot(HaveOccurred())
+	return wh
+}
